#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# Support for fixed physical locations, for example, homes, offices, schools, stores, etc.

import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table
import requests

##################################

console = Console()


@click.group()
def location():
    """Location management"""

@location.command()
@common_cli_params
@click.option("--name", help="Location name", required=True)
@click.option("--desc", help="Location Description", default="")
@click.option("--address", help="Location Address", default="")
@click.option("--image", help="Location Image", default=None)
@click.option("--bg", help="Location Background Image", default=None)
@click.option("--map", help="Location Indoor map URL", default=None)
@click.option("--latitude", "--lat", help="Location Latitude", default=None)
@click.option("--longitude", "--lng", help="Location Longitude", default=None)
@click.option("--altitude", "--alt", help="Location Altitude", default=None)
def add(team, profile, name, desc, address, image, bg, map,
        latitude, longitude, altitude):
    """
    Define a new location
    \f
    Examples:

    $ iot location add --name "..." --address "..."

    """
    try:
        config = preload_config(team, profile)

        payload = {
            "name": name,
            "address": address,
        }
        if desc:
            payload["desc"] = desc
        if latitude:
            payload["geo_lat"] = latitude
        if latitude:
            payload["geo_ng"] = longitude
        if altitude:
            payload["geo_alt"] = altitude
        if image:
            payload["image_url"] = image
        if bg:
            payload["bg_url"] = bg
        if map:
            payload["indoor_map_url"] = map

        response = make_api_request("POST", config, "location", json=payload)
        data = response.json()

        if response.status_code == requests.codes.ok:
            location_id = data["id"]
            table = Table(show_header=True, header_style="green")
            table.add_column("ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_row(location_id, "OK")
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@location.command()
@common_cli_params
@click.option("--name", help="Location name", default=None)
@click.option("--id", help="Location ID", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def list(team, profile, name, id, full):
    """
    List already defined locations
    \f
    Examples:
    \b
    $ iot location list --id "..."
    """
    try:
        config = preload_config(team, profile)

        multi = False

        if name:
            response = make_api_request("GET", config, f"location?name={name}")
        elif id:
            response = make_api_request("GET", config, f"location?id={id}")
        else:
            multi = True
            response = make_api_request("GET", config, f"location?all=true")

        data = response.json()
        if response.status_code == requests.codes.ok:
            if full and not multi:
                show_detail(console, "Location", data)
                return

            table = Table(show_header=True, header_style="green")
            table.add_column("Location ID", style="dim", overflow="flow")
            table.add_column("Name")
            table.add_column("Address")
            table.add_column("Date Created", justify="right")
            if multi:
                for d in data:
                    id = d.get("id", "***")
                    name = d.get("name", "***")
                    address = d.get("address", "***")
                    created = d.get("date_created", "***")
                    table.add_row(id, name, address, format_date(created))
            else:
                id = data.get("id", "***")
                name = data.get("name", "***")
                address = data.get("address", "***")
                created = data.get("date_created", "***")
                table.add_row(id, name, address, format_date(created))
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Location List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@location.command()
@common_cli_params
@click.option("--id", help="Location ID", required=True)
@click.option("--name", help="Location name", required=True)
@click.option("--desc", help="Location Description", default="")
@click.option("--address", help="Location Address", default="")
@click.option("--image", help="Location Image", default=None)
@click.option("--bg", help="Location Background Image", default=None)
@click.option("--map", help="Location Indoor map URL", default=None)
@click.option("--latitude", "--lat", help="Location Latitude", default=None)
@click.option("--longitude", "--lng", help="Location Longitude", default=None)
@click.option("--altitude", "--alt", help="Location Altitude", default=None)
def update(team, profile, id, name, desc, address, image, bg, map,
        latitude, longitude, altitude):
    """
    Update Location attributes
    \f
    Examples:

    $ iot location update --name "..." --address "..."
    """
    try:
        config = preload_config(team, profile)

        payload = {
            "id": id,
        }
        if name:
            payload["name"] = name
        if desc:
            payload["desc"] = desc
        if desc:
            payload["address"] = address
        if image:
            payload["image"] = image
        if bg:
            payload["bg"] = bg
        if map:
            payload["map"] = map
        if latitude:
            payload["lat"] = latitude
        if longitude:
            payload["lng"] = longitude
        if altitude:
            payload["alt"] =  altitude

        url_params = urllib.parse.urlencode(payload)
        url = f"location?{url_params}"
        response = make_api_request('PUT', config, url)
        data = response.json()

        if response.status_code == requests.codes.ok:
            location_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Location ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(location_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@location.command()
@common_cli_params
@click.option("--name", help="Location name", default=None)
@click.option("--id", help="Location ID", default=None)
def delete(team, profile, name, id):
    """Deletes an already-defined location
    \f
    Examples:
    \b
    $ iot location delete --id "..."
    """
    try:
        config = preload_config(team, profile)

        response = None

        if name:
            response = make_api_request("DELETE", config, f"location?name={name}")
        elif id:
            response = make_api_request("DELETE", config, f"location?id={id}")

        data = response.json()
        if response.status_code == requests.codes.ok:
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table

console = Console()

@click.group()
def project():
    """Project management"""

@project.command()
@common_cli_params
@click.option('--name', help='Project name', required=True)
@click.option('--desc', help='Project Description', default="")
@click.option('--template', help='Template Name to use', default="")
@click.option('--template_id', help='Template ID to use', default="")
def add(team, profile, name, desc, template, template_id):
    """
    Define a new Project
    \f
    Example:

    $ iot project add --name "..."
    """
    try:
        config = preload_config(team, profile)

        payload = {
            "project_name": name,
            "desc": desc
        }

        if template_id:
            payload['template_id'] = template_id
        else:
            if template:
                payload['template_name'] = template

        response = make_api_request('POST', config, 'project', json=payload)
        if response:
            data = response.json()
            #
            # This creates the right local directory, if needed
            #
            get_iot_project_dir(config.profile, name, create=True)

            table = Table(show_header=True, header_style="bold green")
            table.add_column("ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            project_id = data.get("id", "")
            status = data.get("status", "ok")
            message = data.get("message", "")
            table.add_row(project_id, status, message)
            console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@project.command()
@common_cli_params
@click.option('--name', '--project', help='Project name', default=None)
@click.option('--id', help='Project ID', default=None)
@click.option('--full/--no-full', help='List Full Data', default=False)
def list(team, profile, name, id, full):
    """
    List defined Projects and show details
    \f
    Examples:

    $ iot project list --name "..."
    """
    try:
        config = preload_config(team, profile)
        multi = False

        if name:
            response = make_api_request('GET', config, f"project?project_name={name}")
        elif id:
            response = make_api_request('GET', config, f"project?project_id={id}")
        else:
            multi = True
            response = make_api_request('GET', config, f"project?all=true")

        data = response.json()

        if response:
            if full and not multi:
                show_detail(console, "Project", data)
                return

            table = Table(show_header=True, header_style="bold green")
            table.add_column("ID", style="dim", overflow="flow")
            table.add_column("Name")
            table.add_column("Date Created", justify="right")
            #print(f"RESULT: Type: {type(data)} - {data}")
            if multi:
                for d in data:
                    table.add_row(d['id'], d['name'], format_date(d['date_created']))
            else:
                table.add_row(data['id'], data['name'], format_date(data['date_created']))
        else:
            status = data.get('status', "")
            message = data.get('message', "")
            table = Table(show_header=True, header_style="red")
            table.add_column("Project List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@project.command()
@common_cli_params
@click.option('--id', help='Project ID', default=None)
@click.option('--name', help='Project name', default=None)
@click.option('--desc', help='Project Deescription', default=None)
def update(team, profile, id, name, desc):
    """
    Updates Project attributes
    \f
    Examples:
    \b
    $ iot project update --name "Project Name" --desc "..."
    """
    try:
        config = preload_config(team, profile)
        params = {}

        if name:
            params["project_name"] = name
        if id:
            params["project_id"] = id
        if desc:
            params["desc"] = desc

        url_params = urllib.parse.urlencode(params)
        url = f"project?{url_params}"
        print(f"Update Project URL: {url}")
        response = make_api_request('PUT', config, url)
        print(f"Response: {response}")
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@project.command()
@common_cli_params
@click.option('--name', help='Project name', default=None)
@click.option('--id', help='Project ID', default=None)
@click.option('--force', '-f', help='Force deletion without confirmation', default=False)
def delete(team, profile, name, id, force):
    """
    Delete an existing Project and all related elements
    \f
    Examples:
    \b
    $ iot project delete --name "..."
    """
    try:
        config = preload_config(team, profile)
        response = None

        if name:
            response = make_api_request('DELETE', config, f"project?project_name={name}")
        elif id:
            response = make_api_request('DELETE', config, f"project?project_id={id}")

        if response.status_code == 418:
            print(f"Invalid parameter. Record not found.")
            data = response.json()
            project_id = data.get('id', "")
            status = data.get('status', "")
            message = data.get('message', "")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Delete ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(project_id, status, message)
            console.print(table)

        elif response.status_code == 200:
            delete_iot_project_dir(config.profile, name)
            data = response.json()
            project_id = data.get('id', "")
            status = data.get('status', "")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_row(project_id, status)
            console.print(table)

            # And now we go try to remove the local .iot/ files

    except Exception as exc:
        print(f"ERROR: {str(exc)}")




#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
import requests
from simpleiot.common.utils import make_api_request, show_detail
from simpleiot.common.config import *
from urllib.parse import unquote
from rich import print
from rich.console import Console
from rich.table import Table
from pathlib import Path
import uuid


console = Console()

@click.group()
def update():
    """OTA Firmware Updates"""

@update.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model Name", default='')
@click.option("--serial", help="Device Serial number", default='')
@click.option("--file", help="Payload file", required=True)
@click.option("--name", "--type", help="Data name", default='')
@click.option("--desc", help="Data Description", default='')
@click.option("--version", help="Update version", required=True)
@click.option("--release_note", help="Inline release notes", default='')
@click.option("--user_data", help="Inline extra user-data", default='')
def upload(team, profile, project, model, serial, file, name, desc, version,
           release_note, user_data):
    """
    Upload binary file for staging firmware update
    \f
    Use this to upload firmware for distribution. Sending updates is a two-step
    process. First is to upload the payload file. Then to submit an update to a list of
    target devices.

    The upload part itself has three parts:
    1) Send a call with the name of payload file.
    2) Get back a pre-signed URL for uploading to S3 using a PUT call.
    3) Call again when upload is finished to indicate upload is done.

    This command encapsulates all three steps.

    The step where the update is sent is captured in a different call.
    """
    try:
        config = preload_config(team, profile)

        raw_url = None
        url = None
        data = None
        firmware_id = None

        upload_file = Path(file)
        if not upload_file.is_file():
            print(f"ERROR: file {file} does not exist")
            exit(1)

        payload = {
            "project": project,
            "file": file,
            "item": "upload"
        }
        if serial:
            payload["serial"] = serial
        elif model:
            payload["model"] = model

        if version:
            payload["version"] = version
        if name:
            payload["name"] = name
        if desc:
            payload["desc"] = desc
        if release_note:
            payload["release_note"] = release_note
        if user_data:
            payload["user_data"] = user_data

        response = make_api_request("POST", config, "update", json=payload)
        data = response.json()

        # This returns a URL that is urlencoded. We need to urldecode it to get the pre-signed URL
        # where we upload the file. But in the third step, we need to return the same urlencoded
        # pre-signed URL, so we need to keep it around.
        #
        if response:
            firmware_id = data.get('firmware_id', None)
            raw_url = data.get('url', None)
            if raw_url:
                url = unquote(raw_url)

            # The URL returned is a pre-signed URL for use with a PUT call.

            if firmware_id and url:
                headers = {'Content-type': 'application/x-binary', 'Slug': file}
                response = requests.put(url, data=open(file, 'rb')) #, headers=headers)
                if response:
                    second_payload = {
                        "project": project,
                        "item": "payload",
                        "firmware_id": firmware_id,
                        "url": raw_url
                    }
                    response = make_api_request("POST", config, "update", json=second_payload)
                    data = response.json()
                    if response:
                        firmware_id = data['firmware_id']

                        table = Table(show_header=True, header_style="green")
                        table.add_column("Project")
                        if serial:
                            table.add_column("Serial")
                        elif model:
                            table.add_column("Model")
                        table.add_column("File")
                        table.add_column("Upload ID", min_width=32, overflow="flow")
                        if serial:
                            table.add_row(project, serial, file, firmware_id)
                        elif model:
                            table.add_row(project, model, file, firmware_id)

                        console.print(table)
                    else:
                        print(f"ERROR: could not finalize upload. Please remove and try again: {str(response.text)}")
            else:
                print("ERROR: invalid data returned from API. Make sure you remove the Firmware record.")
                exit(1)
        else:
            print(f"Error requesting a pre-signed URL to upload firmware: {response.text}")

    except Exception as e:
        print(f"ERROR: {str(e)}")


@update.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--name", help="Session name", default="")
@click.option("--id", "--upload_id", help="Upload ID (from 'upload' command", required=True)
@click.option("--serial", help="Device serial number", default=None)
@click.option("--model", help="Model name", default=None)
def push(team, profile, project, name, id, serial, model):
    """
    Push firmware update to devices
    \f
    Examples:
    \b
    $ iot update push --project "..." --serials "..."
    """
    try:
        config = preload_config(team, profile)

        multi = False
        data = None

        if not (serial or model):
            print("ERROR: need to specify either comma-separated serial numbers or model name for a target device")
            return

        # Using this to verify that it's the right format
        id_check = uuid.UUID(id)

        payload = {
            "project": project,
            "firmware_id": id,
            "item": "session"
        }

        if serial:
            payload["serial"] = serial
        elif model:
                payload["model"] = model

        response = make_api_request("POST", config, "update", json=payload)
        data = response.json()
        if response:
            table = Table(show_header=True, header_style="green")
            table.add_column("Upload ID", style="dim", min_width=32, overflow="flow")
            table.add_column("Project")
            if serial:
                table.add_column("Serial")
            elif model:
                table.add_column("Model")
            table.add_column("Status")

            if serial:
                table.add_row(id, project, serial, "OK")
            if model:
                table.add_row(id, project, model, "OK")
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Push Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")
        traceback.print_exc()

@update.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT")
@click.option("--version", help="Version number", default="")
@click.option("--id", "--firmware_id", help="Upload ID (from 'upload' command")
@click.option("--serial", "--device", help="Device serial number", default=None)
@click.option("--model", help="Model name", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def list(team, profile, project, version, id, serial, model, full):
    """
    List firmware uploaded and ready to be pushed
    """
    try:
        config = preload_config(team, profile)

        multi = False
        data = None

        payload = {
            "item": "upload"
        }

        if id:
            payload["id"] = id
        if project:
            payload["project"] = project
        if serial:
            payload["serial"] = serial
        if model:
            payload["model"] = model

        if not (id or project or serial or model):
            print("ERROR: need to specify one of '--firmware' or '--project and --serial' or '--model' ")
            return

        response = make_api_request("GET", config, "update", params=payload)
        data = response.json()

        if response:
            if full:
                one = data[0]
                show_detail(console, "Update", one)
                return

            table = Table(show_header=True, header_style="green")
            table.add_column("Firmware ID", style="dim", min_width=32, overflow="flow")
            table.add_column("Name", max_width=15)
            table.add_column("Model")
            table.add_column("Serial")
            table.add_column("State")
            table.add_column("Version")

            for one in data:
                update_id = one.get("id", "")
                update_name = one.get("name", "")
                update_serial = one.get("serial", "***")
                update_model = one.get("model", "")
                update_url = one.get("url", "")
                update_state = one.get("state", "")
                update_version = one.get("version", "")
                table.add_row(update_id, update_name, update_model, update_serial, update_state, update_version)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("List Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@update.command()
@common_cli_params
@click.option("--id", "--firmware_id", help="Upload ID (from 'upload' command")
def delete(team, profile, id):
    """
    Delete an existing OTA update file from the staging site
    \f
    Examples:
    \b
    $ iot update delete --id "..."
    """
    try:
        config = preload_config(team, profile)
        response = None

        if name:
            response = make_api_request("DELETE", config, f"update?firmware_id={id}")

        data = response.json()

        if response:
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")



# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# iot firmware -- can be used to:
#    generate: creates templatized code for building firmware
#    build: compile code and build binary (you can pass the wifi ssid info to it)
#    flash: flash binary to device via USB
#
#!/usr/bin/env python

import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

from rich import print
from rich.console import Console
from rich.table import Table
import serial.tools.list_ports
import questionary
import tempfile
import os as ops
import inspect
import zipfile
from pathlib import Path
from shutil import which
import requests
import platform
from simpleiot.cli.buildtool.toolchain import Toolchain
from simpleiot.cli.toolchain import  LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION

#######################

console = Console()


@click.group()
def firmware():
    """Generate sample source"""


@firmware.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT")
@click.option("--serial", help="Device serial", envvar="IOT_DEVICE")
@click.option("--manufacturer", "--brand", help="Manufacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--name", help="Generator name", required=True)
@click.option("--version", help="Firmware version", default=LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION)
@click.option("--wifi_ssid", "--ssid", help="Wifi SSID name", envvar="IOT_WIFI_SSID")
@click.option("--wifi_password", "--password", help="Wifi Password", envvar="IOT_WIFI_PASSWORD")
def generate(team, profile, project, serial, manufacturer, processor, os, name,
             version, wifi_ssid, wifi_password):
    """Generates firmware source
    \f
    Given a defined manufacturer, processor, and OS, generate firmware source that can be compiled
    and uploaded to a device. If the WiFi SSID/Password are specified, they are added to the
    source file.

    \b
    $ iot firmware generate --project=... --serial=... --os=...

    The return data is a .ZIP file with the code.
    """
    try:
        config = preload_config(team, profile)

        show_all = False

        payload = {}
        multi = True

        if project:
            payload["project_name"] = project
        if serial:
            payload["serial"] = serial
        if manufacturer:
            payload["manufacturer"] = manufacturer
        if processor:
            payload["processor"] = processor
        if os:
            payload["os"] = os
        if name:
            payload["generator_name"] = name
        if version:
            payload["version"] = version
        if wifi_ssid:
            payload["wifi_ssid"] = wifi_ssid
        if wifi_password:
            payload["wifi_password"] = wifi_password

        # If no project specified, we show a list
        #
        if not project:
            show_all = True

        if show_all:
            response = make_api_request("GET", config, "firmware")
        else:
            response = make_api_request("POST", config, "firmware", json=payload, stream=True)

        if response.status_code == requests.codes.ok:
            content_type = response.headers['content-type']
            if content_type == 'application/zip':
                content_disp = response.headers['content-disposition']
                filename = re.findall("filename=(.+)", content_disp)[0]

                with open(filename, 'wb') as output:
                    shutil.copyfileobj(response.raw, output)
                del response
                print(f"Done. Code generated into: {filename}")
            else:
                data = response.json()
                table = Table(show_header=True, header_style="green")
                table.add_column("Generator")
                table.add_column("Manufacturer")
                table.add_column("Processor")
                table.add_column("OS")
                table.add_column("File")
                table.add_column("Date Created", justify="right")
                for d in data:
                    name = d.get("name", "***")
                    manufacturer = d.get("manufacturer", "***")
                    processor = d.get("processor", "***")
                    os = d.get("os", "***")
                    zip_url = d.get("zip_url", "***")
                    filename = zip_url.rsplit('/', 1)[1]
                    created = d.get("date_created", "***")
                    table.add_row(name, manufacturer, processor, os, filename, format_date(created))

                console.print(table)
        else:
            data = response.json()
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Generator List Status")
            table.add_column("Message")
            table.add_row(status, message)

            console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


# These are hard-coded for the demo device (M5Stack Core2). Add device VID/PID
# if not recognized or they change their chipset. Run 'lsusb' and add the
# hex value of the USB port into the array below.
#
# NOTE that the fqbn is hard-coded for the M5Stack EduKit device. For other devices,
# you'll need to change this. To get the list of all supported boards, run:
#
#    arduino-cli board listall
#
# The FQBN is listed next to the name, if the board support package is installed for it:
# To get details on the board:
#
#    arduino-cli board details {board-fqbn}
#
# More info here: https://forum.arduino.cc/t/finding-a-fqbn-for-a-device-that-doesnt-come-up-on-board-list/676662
#
FILTER_VID = [0x1a86, 0x10c4]
FILTER_PID = [0x55d4, 0xea60]
FQBN="esp32:esp32:m5stack-core2"

@firmware.command()
@click.option("--base", "--path", help="Base path of toolchain", default=Toolchain.base())
@click.option("--manufacturer", "--brand", help="Manufacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--version", help="Firmware version", default=LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION)
@click.option("--location", help="Install location type",
              type=click.Choice(["local", "local_container", "cloud_container", "cloud_server"], case_sensitive=False),
              default="local")
@click.option("--zip", help="Source zip file")
@click.option("--dir", help="Source directory")
@click.option("--port", help="Serial port to flash device")
def flash(base, manufacturer, processor, os, version, location, zip, dir, port):
    """Build and flash firmware to device
    """
    try:

        # port = "/dev/cu.usbserial-01F9734D"
        port = get_port(port)
        source_dir = None
        clean_on_exit = False

        if not port:
            print(f"ERROR: no --port specified")
            exit(1)

        if zip:
            # Note: in python 3.10, there was an ignore_cleanup_errors parameter added.
            # Once we move the minimum python requirement to 3.10, we can add this to the
            # following constructor.
            # Details: https://docs.python.org/3/library/tempfile.html#tempfile.TemporaryDirectory
            #
            source_dir = tempfile.mkdtemp(prefix="iot")
            # source_dir = source_temp_dir.name
            with zipfile.ZipFile(zip, "r") as zip_file:
                zip_file.extractall(source_dir)
            clean_on_exit = True

        # If they've passed along a directory name instead of a zip file...
        if dir:
            if ops.path.exists(dir):
                source_dir = dir

        # Let's find the sketch_name. Under Arduino, the directory name and the .ino file must match.
        #
        if source_dir:
            directories = ops.scandir(source_dir)
            sketch_name = None
            for entry in directories:
                if entry.is_dir():
                    sketch_name = entry.name
                    break

            if sketch_name:
                sketch_dir = Path(ops.path.join(source_dir, sketch_name))
                # print(f"SKETCH DIR: {sketch_dir}")

                from yaspin import yaspin

                with yaspin(text="Building and Flashing... ", color="green") as spinner:
                    toolchain = Toolchain()
                    command = f"compile -v -u -p {port} --fqbn {FQBN} {sketch_dir}"
                    toolchain.build_and_flash(base, manufacturer, processor, os, version, location, sketch_dir, command)
                    spinner.ok("✅ ")
                    print("Done!")

            else:
                print("ERROR: could not locate sketch root in downloaded project. Invalid template layout.")

        if clean_on_exit:
            import shutil
            # print(f"Cleaning dir: {source_dir}")
            print(f"Cleaning up...")
            shutil.rmtree(source_dir)

    except Exception as e:
        print(f"ERROR flashing device: {str(e)}")



#
# This is hard-set for our demo device, the ESP-32 based M5Stack EduKit. To change it, the various partition sizes
# will need to be calculated and set. This command is here so we can load a very basic animated app onto the device
# after demos. Then when you flash it with a new app, it makes it obvious it's changed.

# NOTE: if we want to re-create the demo, go into Arduino IDE Preferences, and toggle on the 'Show verbose output
# during: compilation' flag. Then build and watch what the command will be for uploading binaries to the device.
# Then replicate those files and that command below. This command is very specific to the M5 Core2 and is here to
# simplify resetting a demo device back into its pre-demo state.
#
# NOTE 2: There is a standalone version of esptool.py on pypi. There was hope that it could be used instead of
# hard-coded/platform specific paths to esptool.py. However, that command did not work when flashing using parameters
# generator by the Arduino toolchain.
#
@firmware.command()
@click.option("--demo", help="Demo directory", default="demo/m5gif")
@click.option("--port", help="Serial port to flash device")
def m5demo(demo, port):
    """Flash a demo Sketch file to the target device.
    """
    try:
        import glob
        from yaspin import yaspin

        tool_path = None

        pkg_dir = Path(ops.path.dirname(ops.path.abspath(inspect.getfile(inspect.currentframe()))))
        demo_path = pkg_dir.parent / demo
        if not ops.path.exists(demo_path):
            print(f"ERROR: could not locate demo directory {demo_path}")
            exit(1)

        opsys = platform.system()
        if opsys == "Darwin":
            tool_path = glob.glob(os.path.expanduser("~/Library/Arduino15/**/esptool"), recursive=True)
        elif opsys == "Windows":
            tool_path = glob.glob(os.path.expanduser("~\\AppData\\Local\\Arduino15\\**\\esptool.exe"), recursive=True)

        if tool_path:
            esptool_path = tool_path[0]
        else:
            print(f"ERROR: could not locate esptool flashing tool. Have you installed the toolchain?")
            exit(1)

        port = get_port(port)
        print(f"Writing via Port: {port}")
        if port:
            if Path(demo_path).is_dir():
                chip = "esp32"
                boot_app0 = demo_path / "boot_app0.bin"
                bootloader = demo_path / "m5gif.ino.bootloader.bin"

                bin_file = demo_path / "m5gif.ino.bin"
                partition_bin_file = demo_path / "m5gif.ino.partitions.bin"

                flash_size = "16MB"
                flash_freq = "80m"
                baud_rate = "921600"

                command = f"{esptool_path} --chip {chip} --port {port} --baud {baud_rate} " \
                          f"--before default_reset --after hard_reset " \
                          f"write_flash " \
                          f"-z --flash_mode dio --flash_freq {flash_freq} " \
                          f"--flash_size {flash_size} " \
                          f"0x1000 {bootloader} " \
                          f"0x8000 {partition_bin_file} " \
                          f"0xe000 {boot_app0} " \
                          f"0x10000 {bin_file}"
                # print(f"Command: {command}")

                with yaspin(text="Flashing... ", color="green") as spinner:
                    ops.system(command)
                    spinner.ok("✅ ")

            print("Done!")

    except Exception as e:
        print(f"ERROR flashing device: {str(e)}")


def get_list_of_serial_ports():
    devices = []
    for item in serial.tools.list_ports.comports():
        # item_json = json.dumps(item.__dict__, indent=2)
        # print(f"Got port: {item_json}")
        if item.vid in FILTER_VID and item.pid in FILTER_PID:
            device = {
                   "name": item.name,
                   "port": item.device
            }
            devices.append(device)
    return devices

#
# If a port has been defined on the command-line we use that. Otherwise we scan for what's available
# on the system and if more than one, we ask the user which one to use.
#
def get_port(port):
    if not port:
        devices = get_list_of_serial_ports()
        if len(devices) == 0:
            print("ERROR: no M5 device found. Please install driver and plug device into USB and try again.")
            exit(1)

        # If we only have one port, we just use it. Otherwise we ask to pick which one to use.
        #
        if len(devices) == 1:
            port = devices[0]['port']
        else:
            device_list = []
            for d in devices:
                device_list.append(d['name'])

            device_name = questionary.select("Choose USB port to use: ", choices=device_list).ask()
            if device_name:
                for dev in devices:
                    if device_name == dev['name']:
                        port = dev['port']
    return port


def does_cli_exist(name):
    opsys = platform.system()
    if opsys == "Darwin":
        cli = which(name)
    elif opsys == "Windows":
        cli = ops.path.expanduser("~\\Downloads\\arduino-cli.exe")
    return cli



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This is a test script for the CLI, using the click CliRunner tool. To run, use
# pytest. Note that the following environment variables should be set in order for
# the test to work:
#
# IOT_TEAM: team name to test against
# IOT_USERNAME: userid of valid account
# IOT_PASSWORD: password for valid account
#

import os
from click.testing import CliRunner
from simpleiot.iot import iotcli

def check_auth():
    """
    Basic check that the environment variables everty command needs are set.
    """
    runner = CliRunner()
    assert runner

    team = os.getenv("IOT_TEAM")
    username = os.getenv("IOT_USERNAME")
    password = os.getenv("IOT_PASSWORD")
    assert team
    assert username
    assert password

    return runner, team, username, password


def check_login():
    """
    Performs standard login to make sure we have an auth token to use for
    subsequent interactions.
    """
    runner, team, username, password = check_auth()
    result = runner.invoke(iotcli, ['auth', 'login', f"--username={username}", f"--password={password}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output
    return runner


def test_help():
    """
    Basic test to see that the command is available
    """
    runner, team, username, password = check_auth()
    result = runner.invoke(iotcli, ['--help'])
    assert result.exit_code == 0


def test_auth_login_logout():
    """
    Login/Logout with (hopefully) valid user.
    """
    runner = check_login()
    result = runner.invoke(iotcli, ['auth', 'logout', "--clear"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output


def test_bad_auth_login_logout():
    """
    Test for invalid username/password combos. These tests should fail.
    """
    runner, team, username, password = check_auth()

    bad_username = "BAD_USER_XXX"
    bad_password = "BAD_PASSWORD_XXX"

    result = runner.invoke(iotcli, ['auth', 'login', f"--username={bad_username}", f"--password={password}"])
    assert result.exit_code == 0
    assert "ERROR" in result.output

    result = runner.invoke(iotcli, ['auth', 'login', f"--username={username}", f"--password={bad_password}"])
    assert result.exit_code == 0
    assert "ERROR" in result.output

    result = runner.invoke(iotcli, ['auth', 'login', f"--username={bad_username}", f"--password={bad_password}"])
    assert result.exit_code == 0
    assert "ERROR" in result.output

    result = runner.invoke(iotcli, ['auth', 'logout', "--clear"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output


def test_project_list_all():
    """
    Get a list of all projects. We check for the word "ERROR" in the return.
    """
    runner = check_login()
    result = runner.invoke(iotcli, ['project', 'list'])
    assert result.exit_code == 0
    assert "ERROR" not in result.output

def test_project_add_then_delete():
    """
    Test for adding a new blank project, verifying that the name is returned when listing
    projects, then cleaning it up. Due to the way tables are aligned some of the names may
    be truncated, so we truncate the test names to make sure we don't accidentally return
    a false negative.
    """
    project_name = "test_plain_proj"
    runner = check_login()
    result = runner.invoke(iotcli, ['project', 'add', f"--name={project_name}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output

    result = runner.invoke(iotcli, ['project', 'list', f"--name={project_name}"])
    assert result.exit_code == 0
    assert project_name[1:6] in result.output
    assert "ERROR" not in result.output
    assert project_name[1:6] in result.output

    result = runner.invoke(iotcli, ['project', 'delete', f"--name={project_name}"])
    assert "ERROR" not in result.output

    result = runner.invoke(iotcli, ['project', 'list', f"--name={project_name}"])
    assert "error" in result.output


def test_project_create_template_then_delete():
    """
    This test creates a project based on a built-in template, then checks to make sure
    all the sub-components like Model and Datatype are present. It then cleans up the project
    And checks to make sure the values are no longer present.

    If this test fails, the project/model/datatypes may have to be manually cleaned up.
    """
    project_name = "test_template_proj"
    template_name = "HelloWorldM5"
    model_name = "HelloWorldModel"

    runner = check_login()

    result = runner.invoke(iotcli, ['project', 'add', f"--name={project_name}", f"--template={template_name}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output

    result = runner.invoke(iotcli, ['project', 'list', f"--name={project_name}"])
    assert result.exit_code == 0
    assert project_name[1:6] in result.output
    assert "ERROR" not in result.output

    result = runner.invoke(iotcli, ['model', 'list', f"--project={project_name}", f"--model={model_name}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output
    assert project_name[1:6] in result.output
    assert model_name[1:6] in result.output

    result = runner.invoke(iotcli, ['datatype', 'list', f"--project={project_name}", f"--model={model_name}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output
    assert project_name[1:6] in result.output
    assert model_name[1:6] in result.output
    assert "button" in result.output
    assert "color" in result.output

    result = runner.invoke(iotcli, ['project', 'delete', f"--name={project_name}"])
    assert result.exit_code == 0
    assert "ERROR" not in result.output

    result = runner.invoke(iotcli, ['datatype', 'list', f"--project={project_name}", f"--model={model_name}"])
    assert "error" in result.output

    result = runner.invoke(iotcli, ['model', 'list', f"--project={project_name}", f"--model={model_name}"])
    assert "error" in result.output

    result = runner.invoke(iotcli, ['project', 'list', f"--name={project_name}"])
    assert "error" in result.output



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table

##################################

console = Console()


@click.group()
def model():
    """Manage Models"""


@model.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--name", "--model", help="Model name", required=True)
@click.option("--desc", help="Model Description", default="")
@click.option("--revision", help="Model Revision", default="")
@click.option("--display_name", help="Model Display Name", default="")
@click.option("--display_order", help="Model Display Order", default="0")
@click.option("--image", help="Model Image URL", default="")
@click.option("--icon", help="Model Icon Image URL", default="")
@click.option("--require_position", help="Model Requires Position Data", is_flag=True)
@click.option("--type", help="Model Type", default="device")
@click.option("--security", help="Model Security", default="device")
@click.option("--storage", help="Model Storage", default="none")
@click.option("--protocol", help="Model Protocol", default="mqtt")
@click.option("--connection", help="Model Connection", default="direct")
@click.option("--ml", help="Model ML", default="none")
@click.option("--hw_version", help="Model Hardware Version")
@click.option('--template', help='Model Template Name to use', default="")
@click.option('--template_id', help='Model Template ID to use', default="")
def add(team, profile, project, name, desc, revision,
        display_name, display_order, image, icon, require_position,
        type, security, storage, protocol, connection, ml, hw_version,
        template, template_id):
    """
    Define a new device Model
    \f
    Examples:

    $ iot model add --name "..." --project "..."

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    Doing so lets you skip the --project flag.
    """
    try:
        config = preload_config(team, profile)

        payload = {
            "project_name": project,
            "name": name
        }
        if desc:
            payload["desc"] = desc
        if revision:
            payload["revision"] = revision
        if display_name:
            payload["display_name"] = display_name
        if display_order:
            payload["display_order"] = display_order
        if image:
            payload["image_url"] = image
        if icon:
            payload["icon_url"] = icon
        if require_position:
            payload["require_position"] = require_position
        if type:
            payload["type"] = type
        if protocol:
            payload["protocol"] = protocol
        if security:
            payload["security"] = security
        if storage:
            payload["storage"] = storage
        if connection:
            payload["connection"] = connection
        if ml:
            payload["ml"] = ml
        if hw_version:
            payload["hw_version"] = hw_version

        if template_id:
            payload['template_id'] = template_id
        else:
            if template:
                payload['template'] = template

        response = make_api_request("POST", config, "model", json=payload)
        data = response.json()

        if response:
            #
            # This creates the right local model cache directory, if needed
            #
            get_iot_model_dir(config.profile, project, name, create=True)

            table = Table(show_header=True, header_style="green")
            table.add_column("ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            project_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table.add_row(project_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Model Add Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@model.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--name", "--model", help="Model name", default=None)
@click.option("--id", help="Model ID", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def list(team, profile, project, name, id, full):
    """
    List defined Models and show details
    \f
    Examples:
    \b
    $ iot model list --project "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        multi = False

        if name:
            response = make_api_request("GET", config, f"model?project={project}&model={name}")
        elif id:
            response = make_api_request("GET", config, f"model?project_name={project}&model_id={id}")
        else:
            multi = True
            response = make_api_request("GET", config, f"model?project_name={project}&all=true")

        data = response.json()
        if response:
            if full and not multi:
                show_detail(console, "Model", data)
                return

            table = Table(show_header=True, header_style="green")
            table.add_column("Model ID", style="dim", overflow="flow")
            table.add_column("Project")
            table.add_column("Name")
            table.add_column("Type")
            table.add_column("Date Created", justify="right")
            if multi:
                for d in data:
                    id = d.get("id", "***")
                    project = d.get("project", "***")
                    model = d.get("model", "***")
                    model_type = d.get("type", "***")
                    created = d.get("date_created", "***")
                    table.add_row(id, project, model, model_type, format_date(created))
            else:
                id = data.get("id", "***")
                project = data.get("project", "***")
                model = data.get("model", "***")
                model_type = data.get("type", "***")
                created = data.get("date_created", "***")
                table.add_row(id, project, model, model_type, format_date(created))
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Model List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@model.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--id", help="Model ID")
@click.option("--name", "--model", help="Model name")
@click.option("--desc", help="Model Description")
@click.option("--revision", help="Model Revision")
@click.option("--display_name", help="Model Display Name")
@click.option("--display_order", help="Model Display Order")
@click.option("--image", help="Model Image URL")
@click.option("--icon", help="Model Icon Image URL")
@click.option("--require_position", help="Model Requires Position Data", is_flag=True)
@click.option("--type", help="Model Type")
@click.option("--security", help="Model Security")
@click.option("--storage", help="Model Storage")
@click.option("--protocol", help="Model Protocol")
@click.option("--connection", help="Model Connection")
@click.option("--ml", help="Model ML", default=None)
@click.option("--hw_version", help="Model Hardware Version")
def update(team, profile, project, id, name, desc, revision,
           display_name, display_order, image, icon, require_position,
           type, security, storage, protocol, connection, ml, hw_version):
    """
    Update Model attributes
    \f
    NOTE: a model's project can not be modified.
    To change it, delete it then re-add with the new project.

    Examples:

    $ iot model update --name "..." --project "..." --image

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    Doing so lets you skip the --project flag.
    """
    try:
        config = preload_config(team, profile)

        if not name and not id:
            print(f"Error: model 'name' or 'id' must be specified")
            exit(1)

        payload = {
            "project_name": project,
        }
        if name:
            payload["model"] = name
        elif id:
            payload["model_id"] = id

        if desc:
            payload["desc"] = desc
        if revision:
            payload["revision"] = revision
        if display_name:
            payload["display_name"] = display_name
        if display_order:
            payload["display_order"] = display_order
        if image:
            payload["image_url"] = image
        if icon:
            payload["icon_url"] = icon
        if require_position:
            payload["require_position"] = require_position
        if type:
            payload["type"] = type
        if protocol:
            payload["protocol"] = protocol
        if connection:
            payload["connection"] = connection
        if security:
            payload["security"] = security
        if storage:
            payload["storage"] = storage
        if ml:
            payload["ml"] = ml
        if hw_version:
            payload["hw_version"] = hw_version

        url_params = urllib.parse.urlencode(payload)
        url = f"model?{url_params}"
        response = make_api_request('PUT', config, url)
        data = response.json()

        if response:
            model_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Model Update ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(model_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")


@model.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--name", "--model", help="Model name", default=None)
@click.option("--id", help="Project ID", default=None)
def delete(team, profile, project, name, id):
    """
    Delete an existing Model and all related elements
    \f
    Examples:
    \b
    $ iot model delete --project "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        response = None

        if name:
            response = make_api_request("DELETE", config, f"model?project_name={project}&model={name}")
        elif id:
            response = make_api_request("DELETE", config, f"model?project_name={project}&model_id={id}")

        data = response.json()

        if response:
            delete_iot_model_dir(config.profile, project, name)
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as exc:
        print(f"ERROR: {str(exc)}")




#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This command manages build toolchains. It is here to allow
# the build toolchain to be easily installed. It currently supports local install of
# the arduino-cli for ESP32, but is intended so it can be extended to Docker-based
# local as well as remote Cloud-based installations.
#
# It is also intended to be cross-platform (Mac/Win)
# The operations available are:
#
# - toolchain install: install one or more versions of the toolchain.
# - toolchain update: update to the latest version of the toolchain.
# - toolchain uninstall: remove.
# - toolchain list: list all the toolchains available for building.
#

import click
from simpleiot.common.utils import *
from simpleiot.common.config import *
from simpleiot.cli.buildtool.toolchain import Toolchain
from simpleiot.cli.buildtool.toolchainbase import ToolchainLocation

from rich import print
from rich.console import Console
from rich.table import Table
import serial.tools.list_ports
import questionary
import tempfile
import os
import zipfile
from pathlib import Path
from shutil import which
import requests
import platform
import tempfile


#
# NOTE: the toolchain directory starts with an underline. When listing all the teams installed
# we specifically exclude directory names that start with and underline. This way, we can find all local teams
# simply by scanning the contents of the ~/.simpleiot directory instead of managing a local registry which can
# fall out of sync if someone messes with the directory.
#
console = Console()
toolchain_class = Toolchain()

# Update this with latest ESP32 toolchain version getting installed.
# For non-Arduino toolchains, we will need a central registry.
#
LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION="3.3.0"


@click.group()
def toolchain():
    """Project Template management"""

@toolchain.command()
@click.option("--base", "--path", help="Base path of toolchain", default=Toolchain.base())
@click.option("--alias", "--nickname", help="Toolchain alias", default="esp32arloc")
@click.option("--manufacturer", "--brand", help="Manufacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--version", help="Toolchain version", default=LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION)
@click.option("--location", help="Install location type",
              type=click.Choice(["local", "local_container", "cloud_container", "cloud_server"], case_sensitive=False),
              default="local")
def install(base, alias, manufacturer, processor, os, version, location):
    """
    Install the toolchain given manufacturer, processor, OS, and version.
    Defaults to the DevKit: Espressif:ESP32:Arduino:latest:Local.

    Location can be:
      - **local** or **native**: Installed on the local filesystem.
      - **local_container**: Installed as a pre-built Docker image. Assumes Docker desktop is installed.
      - **cloud_container**: Installed in your own account as an ECS container.
      - **cloud_server**: Pointed at a centralized development Cloud installation.

    Currently only the local option is supported.

    The `--team` parameter needs to be specified since we keep track of installations
    inside the local team configuration file.

    If a toolchain is already installed, this command will fail. To reinstall you have to first
    uninstall that version and then reinstall it using this command.

    \f
    Example:

    $ not toolchain install
    """
    global toolchain_class
    try:
        location_enum = ToolchainLocation(location)

        from yaspin import yaspin

        with yaspin(text="Installing... ", color="green") as spinner:
            if alias:
                result = toolchain_class.install_alias(base, alias)
                # print(f"Install result: {result}")
            else:
                toolchain_class.install(base, manufacturer, processor, os, version, location_enum)

            toolchain_class.reset(base, manufacturer, processor, os, version, location_enum)
            spinner.ok("✅ ")

    except Exception as e:
        print(f"ERROR: {str(e)}")


@toolchain.command()
@click.option("--base", "--path", help="Base path of toolchain", default=Toolchain.base())
@click.option("--alias", "--nickname", help="Toolchain alias", default=None)
@click.option("--manufacturer", "--brand", help="Manufacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--version", help="Toolchain current version", default=None)
@click.option("--to", help="Toolchain desired version", default=None)
def update(base, alias, manufacturer, processor, os, version, to):
    """
    Update installed toolchain to given version.
    """
    global toolchain_class
    try:
        print("Not available yet.")

    except Exception as e:
        print(f"ERROR: {str(e)}")


@toolchain.command()
@click.option("--base", "--path", help="Base path of toolchain", default=Toolchain.base())
@click.option("--alias", "--nickname", help="Toolchain alias")
@click.option("--manufacturer", "--brand", help="Maunfacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--version", help="Toolchain current version", default=LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION)
@click.option("--location", help="Install location type",
              type=click.Choice(["local", "local_container", "cloud_container", "cloud_server"], case_sensitive=False),
              default="local")
def uninstall(base, alias, manufacturer, processor, os, version, location):
    """
    Remove installed toolchain.
    """
    global toolchain_class
    try:
        location_enum = ToolchainLocation(location)

        if alias:
            result = toolchain_class.uninstall(base, alias)
        else:
            result = toolchain_class.uninstall(base, manufacturer, processor, os, version, location_enum)
    except Exception as e:
        print(f"ERROR: {str(e)}")

#
# NOTE: the version number must match the default version of the toolchain installed for the platform.
# If it isn't installed, we'll have to go find it from a central registry. For now, however, we default to
# the current latest version.
#

@toolchain.command()
@click.option("--base", "--path", help="Base path of toolchain", default=Toolchain.base())
@click.option("--manufacturer", "--brand", help="Maunfacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
@click.option("--version", help="Toolchain current version", default=LATEST_ARDUINO_ESP32_TOOLCHAIN_VERSION)
@click.option("--location", help="Install location type",
              type=click.Choice(["local", "local_container", "cloud_container", "cloud_server"], case_sensitive=False),
              default="local")
def reset(base, manufacturer, processor, os, version, location):
    """
    Remove installed toolchain.
    """
    global toolchain_class
    try:
        location_enum = ToolchainLocation(location)
        result = toolchain_class.reset(base, manufacturer, processor, os, version, location_enum)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@toolchain.command()
@click.option("--manufacturer", "--brand", help="Maunfacturer name", default="espressif")
@click.option("--processor", "--cpu", help="Processor type", default="esp32")
@click.option("--os", help="Operating system name", default="arduino")
def list(manufacturer, processor, os):
    """
    List available and/or installed toolchains on this machine. This is loaded out of the
    .simpleiot/toolchain.json file.
    """
    global toolchain_class
    try:
        print("NotImpl")

    except Exception as e:
        print(f"ERROR: {str(e)}")


@toolchain.command()
def available():
    """
    List available and/or installed toolchains.

    This scans through the list of registered toolchain subclasses and shows what can be
    installed.
    """
    global toolchain_class
    try:
        toolchain_list = toolchain_class.list_available()
        table = Table(show_header=True, header_style="green")
        table.add_column("Alias", style="bold", overflow="flow")
        table.add_column("Name", style="bold", overflow="flow")
        table.add_column("Mfr", style="bold", overflow="flow")
        table.add_column("Proc", style="bold", overflow="flow")
        table.add_column("OS", style="bold", overflow="flow")
        table.add_column("Version", style="bold", overflow="flow")
        table.add_column("Location", style="bold", overflow="flow")

        for _, tool in toolchain_list.items():
            table.add_row(tool.alias,
                          tool.name,
                          tool.manufacturer,
                          tool.processor,
                          tool.opsys,
                          tool.version,
                          tool.location.name)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")




# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This is the base class for toolchain installer/uninstallers. Derived classes
# will then provide ways to install toolchains in a way that can be invoked by
# the firmware to compile and flash a device.
#
# We could have implemented this as a config file, but some toolchains require
# checking for conditions, so having it be done in code would allow inclusion
# of code to make sure everything is set up as needed.
#
from .toolchain import Toolchain
from .toolchainbase import ToolchainLocation, ToolChainBase, ToolchainLocation
from simpleiot.common.utils import *
from simpleiot.common.config import *
import os
from pathlib import Path
import platform
import traceback

ARDUINO_TOOLCHAIN_CONFIG = "~/Library/Arduino15/arduino-cli.yaml"


class ToolchainESP32Arduino_1_0_0(ToolChainBase):
    def __init__(self):
        version_list = self.get_installed_toolchain_version()

        super(ToolchainESP32Arduino_1_0_0, self).__init__(
            "esp32arloc",
            "Espressif Esp32 Arduino Local",
            "Local Arduino-CLI install for ESP32 ArduinoCore",
            "espressif",
            "esp32",
            "arduino",
            version_list)
        opsys = platform.system()
        if opsys == "Darwin":
            self.executable = "arduino-cli"
        elif opsys == "Windows":
            self.executable = "arduino-cli.exe"
        elif opsys == "Linux":
            self.executable = "arduino-cli"
        else:
            print(f"ERROR: operating system not supported")
            exit(1)


    def install_windows(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            self.location = location
            if location == ToolchainLocation.LOCAL:
                print(f"Local install {self.name} - {self.desc} for Windows")
                source_path = "https://downloads.arduino.cc/arduino-cli/arduino-cli_latest_Windows_64bit.zip"
                temp_path = Path(tempfile.mkdtemp())
                zip_file = "arduino-cli.zip"
                install_exe = self.exec_path(base)
                download_path = temp_path / zip_file

                exec_command = "powershell Set-PSDebug -Trace 0 &" \
                               f"powershell 'If (Test-Path -Path \"{install_exe}\") {{ Remove-Item -Path \"{install_exe}\" -Force }}' &" \
                               f"cd {temp_path} & " \
                               f"powershell Invoke-WebRequest \"{source_path}\" -OutFile \"{download_path}\" & "\
                               f"powershell -NoLogo -NoProfile -Command Expand-Archive \"{zip_file}\" & "\
                               f"powershell Move-Item arduino-cli/arduino-cli.exe \"{install_exe}\" -Force"
                self._exec(exec_command)
                print(f"Done.")
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)

        except Exception as e:
            print(f"ERROR: could not install arduino-cli: {str(e)}")
            exit(1)

    # For Mac,we can use Homebrew, but then we can't install multiple versions of toolchain on a single
    # system. For Arduino, we can use the install script and install it in separate directories.
    #
    def install_mac_with_brew(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            self.location = location
            if location == ToolchainLocation.LOCAL:
                print(f"Local install {self.name} - {self.desc} for Mac")
                if not self._app_exists("arduino-cli"):
                    if self._app_exists("brew"):
                        self._exec("brew install arduino-cli")
                    else:
                        print("ERROR: Homebrew (https://brew.sh/) has to be installed.")
                        exit(1)

                if not self._file_exists("~/Library/Arduino15/arduino-cli.yaml"):
                    self._exec("arduino-cli config init --overwrite")

                self.reset_mac(location=location)
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)

        except Exception as e:
            print(f"ERROR: could not install toolchain. {str(e)}")
            exit(1)

    def install_mac(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            self.location = location
            if location == ToolchainLocation.LOCAL:
                print(f"Local install {self.name} - {self.desc} for Mac")

                install_exe = self.exec_path(base)
                if self._file_exists(install_exe):
                    print(f"ERROR: Toolchain app already exists at path: {install_exe}")
                    exit(1)
                else:
                    self._exec(f"curl -fsSL https://raw.githubusercontent.com/arduino/arduino-cli/master/install.sh | "
                               f"BINDIR={base} sh")

            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR: could not install toolchain. {str(e)}")
            exit(1)

    def uninstall_windows(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local uninstall {self.name} - {self.desc} for Windows")

                install_exe = self.exec_path(base)
                delete_command = "powershell Set-PSDebug -Trace 0 &" \
                                 f"powershell 'If (Test-Path -Path \"{install_exe}\") {{ Remove-Item -Path \"{install_exe}\" -Force }}' &"\
                                 f"powershell 'If (Test-Path -Path \"{base}\") {{ Remove-Item -Path \"{base}\" -Force }}'"

                self._exec(delete_command)
                print(f"File {base} removed.")
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR uninstalling tool: {str(e)}")
            exit(1)

    def uninstall_mac_with_brew(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local uninstall {self.name} - {self.desc} for Mac")

                if self._app_exists("arduino-cli"):
                    if self._app_exists("brew"):
                        self._exec("brew uninstall arduino-cli --force --quiet")
                    else:
                        print("ERROR: Homebrew (https://brew.sh/) has to be installed.")
                        exit(1)
                else:
                    print(f"ERROR: arduino-cli not found in path")
        except Exception as e:
            print(f"ERROR uninstalling tool: {str(e)}")
            exit(1)

    def uninstall_mac(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local uninstall {self.name} - {self.desc} for Mac")

                install_exe = self.exec_path(base)
                if self._file_exists(install_exe):
                    import shutil
                    shutil.rmtree(base)
                else:
                    print(f"ERROR: toolchain executable not found in path: {install_exe}")
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR uninstalling tool: {str(e)}")
            exit(1)


    def reset_windows(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local reset to factory settings for {self.name} for Windows")
                install_exe = self.exec_path(base)

                if self._file_exists(install_exe):
                    self._exec(f"{install_exe} config init --overwrite")
                    self._exec(f"{install_exe} config set board_manager.additional_urls https://raw.githubusercontent.com/espressif/arduino-esp32/gh-pages/package_esp32_index.json")
                    self._exec(f"{install_exe} config set library.enable_unsafe_install true")
                    self._exec(f"{install_exe} core update-index ")
                    self._exec(f"{install_exe} core install esp32:esp32")
                    self._exec(f"{install_exe} lib install ArduinoJson")
                    self._exec(f"{install_exe} lib install ArduinoMqttClient")
                    self._exec(f"{install_exe} lib install FastLED")
                    self._exec(f"{install_exe} lib install TinyGPSPlus-ESP32")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/M5Core2.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENV.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENCODER.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/aws-samples/arduino-aws-greengrass-iot.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/awslabs/simpleiot-arduino.git")
                    print("Done: arduino-cli configured")
                else:
                    print(f"Error: arduino-cli not found at {install_exe}")
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR resetting tool to factory default: {str(e)}")
            exit(1)


    def reset_mac_with_brew(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local reset to factory-settings for {self.name} for Mac")

                if not self._app_exists("arduino-cli"):
                    print("ERROR: arduino-cli is not installed")
                    exit(1)

                if self._file_exists(ARDUINO_TOOLCHAIN_CONFIG):
                    print(f"WARNING: Arduino toolchain configuration file already exists.")
                    print(f"Please move {ARDUINO_TOOLCHAIN_CONFIG} then re-run install command.")
                    exit(1)

                install_exe = "arduino-cli"

                self._exec(f"{install_exe} config set board_manager.additional_urls https://raw.githubusercontent.com/espressif/arduino-esp32/gh-pages/package_esp32_index.json")
                self._exec(f"{install_exe} config set library.enable_unsafe_install true")
                self._exec(f"{install_exe} core update-index ")
                self._exec(f"{install_exe} core install esp32:esp32")
                self._exec(f"{install_exe} lib install ArduinoJson")
                self._exec(f"{install_exe} lib install ArduinoMqttClient")
                self._exec(f"{install_exe} lib install FastLED")
                self._exec(f"{install_exe} lib install TinyGPSPlus-ESP32")
                self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/M5Core2.git")
                self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENV.git")
                self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENCODER.git")
                self._exec(f"{install_exe} lib install --git-url https://github.com/aws-samples/arduino-aws-greengrass-iot.git")
                self._exec(f"{install_exe} lib install --git-url https://github.com/awslabs/simpleiot-arduino.git")
                print(f"Done: {install_exe} configured")
            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR: Could not reset {install_exe}: {str(e)}")
            exit(1)

    def reset_mac(self, base=Toolchain.base(), location=ToolchainLocation.LOCAL):
        try:
            if location == ToolchainLocation.LOCAL:
                print(f"Local reset to factory-settings for {self.name} for Mac")

                install_path = self.install_path(base)
                install_exe = self.exec_path(install_path)
                if install_exe.exists():
                    if not self._file_exists(ARDUINO_TOOLCHAIN_CONFIG):
                        self._exec(f"{install_exe} config init --overwrite")

                    self._exec(
                        f"{install_exe} config set board_manager.additional_urls https://raw.githubusercontent.com/espressif/arduino-esp32/gh-pages/package_esp32_index.json")
                    self._exec(f"{install_exe} config set library.enable_unsafe_install true")
                    self._exec(f"{install_exe} core update-index")
                    self._exec(f"{install_exe} core install esp32:esp32")
                    self._exec(f"{install_exe} lib install ArduinoJson")
                    self._exec(f"{install_exe} lib install ArduinoMqttClient")
                    self._exec(f"{install_exe} lib install FastLED")
                    self._exec(f"{install_exe} lib install TinyGPSPlus-ESP32")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/M5Core2.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENV.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/m5stack/UNIT_ENCODER.git")
                    self._exec(
                        f"{install_exe} lib install --git-url https://github.com/aws-samples/arduino-aws-greengrass-iot.git")
                    self._exec(f"{install_exe} lib install --git-url https://github.com/awslabs/simpleiot-arduino.git")
                    print(f"Done: {install_exe} configured")
                else:
                    print(f"ERROR: build tool not found in local path: {base}.\nPlease uninstall then re-install.")
                    exit(1)

            else:
                print(f"ERROR: non-local toolchains not supported.")
                exit(1)
        except Exception as e:
            print(f"ERROR: Could not reset {install_exe}: {str(e)}")
            exit(1)

    #
    # Parameters will be passed to compiler and flash tool
    #
    def build(self, base, dirpath, command):
        exec = self.exec_path(base)
        full_command = f"{exec} {command}"
        os.chdir(dirpath)
        print(f"DIR: {dirpath}")
        print(f"EXECUTING: {full_command}")
        self._invoke_unbuffered(full_command)
        return True

    def flash(self, base, dirpath, command):
        exec = self.exec_path(base)
        full_command = f"{exec} {command}"
        print(f"DIR: {dirpath}")
        print(f"EXECUTING: {full_command}")
        os.chdir(dirpath)
        self._invoke_unbuffered(full_command)
        return True

    def build_and_flash(self, base, dirpath, command):
        install_path = self.install_path(base)
        exec = self.exec_path(install_path)
        full_command = f"{exec} {command}"
        # print(f"DIR: {dirpath}")
        # print(f"EXECUTING: {full_command}")
        # self._invoke_unbuffered(full_command)
        self._exec(full_command)
        return True


    # This method looks for the local toolchain installation. If it's there, it returns it.
    # Note that this may be different than the version returned by the tool itself.
    # We need this version so at runtime, the flasher tool can be located.
    # Each platform will have a different way of determining it.
    # If the tool hasn't been installed yet, we just return "***".
    #
    def get_installed_toolchain_version(self, install_path=None):
        result = "***"
        try:
            import glob
            tool_path = None

            opsys = platform.system()
            if opsys == "Darwin":
                tool_path = glob.glob(os.path.expanduser("~/Library/Arduino15/**/esptool"), recursive=True)
            elif opsys == "Windows":
                tool_path = glob.glob(os.path.expanduser("~\\AppData\\Local\\Arduino15\\**\\esptool.exe"),
                                      recursive=True)

            version_list = []
            if tool_path:
                for one in tool_path:
                    one_list = one.split(os.sep)
                    version = one_list[-2]
                    version_list.append(version)
                result = ", ".join(version_list)
            return result
        except Exception as e:
            print(f"ERROR flashing device: {str(e)}")




#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
try:
    import boto3
    from botocore.exceptions import ClientError
    from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient
except Exception as e:
    print("ERROR: Have you installed from dist or run 'source venv/bin/activate' to initialize dev environment?")
    exit(1)

import datetime
import json
import sys
import signal
import time
import requests
from requests.exceptions import RequestException
from rich import print
from rich.table import Table
import arrow
import stat
import string
import random
import os
from .config import *

#
# This needs to change so it uses different parameters depending on whether the
# back-end support COGNITO authentication or IAM auth (when SSO is used).
#
def make_api_request(method, config, command, **kwargs):
    """Makes API request and handles connection errors"""
    try:
        url = f"{config.api_endpoint}v1/{command}"

        if config.use_sso:
            access_key = get_stored_access_key(config)
            secret_key = get_stored_access_secret(config)
            session_token = get_stored_session_token(config)
            api_endpoint = config.api_endpoint
            region = config.region

            from requests_aws4auth import AWS4Auth
            auth = AWS4Auth(access_key, secret_key, region, 'execute-api',
                            session_token=session_token)

            response = requests.request(method, url, auth=auth, **kwargs)
        else:

            token = get_stored_api_token(config)
            if not token:
                print("ERROR: not logged in. Please run 'iot auth login' to login first.")
                exit(1)

            headers = {
                "Authorization": token
            }

            response = requests.request(method, url, headers=headers, **kwargs)

        if response.status_code != requests.codes.ok and \
           response.status_code != requests.codes.created:
            if response.status_code == requests.codes.unauthorized:
                print("ERROR. Unauthorized. Make sure you have logged-in.")
                exit(1)
            elif response.status_code == requests.codes.forbidden:
                print("ERROR: Request Forbidden. Username does not have permission.")
                print(response.json()['message'])
                exit(1)
            elif response.status_code != 418:
                try:
                    message = response.json()['message']
                    print(f"ERROR: {message}")
                except:
                    print(response.text,
                          '(HTTP error ' + str(response.status_code) + ')')
                    exit()
        return response
    except RequestException as error:
        print('[ Error connecting to the CLI API server:', config.api_endpoint, \
              ']',
              '(' + str(error) + ')')
        exit()


def show_detail(console, name, data):
    table = Table(show_header=True, header_style="green")
    table.add_column("Key", style="dim", overflow="flow")
    table.add_column("Value")
    for key, value in data.items():
        if isinstance(value, list):
            try:
                json_value = json.dumps(value, indent=2)
                value = json_value
            except:
                value = ", ".join(value)
        table.add_row(key, str(value))
    console.print(table)


def write_qr_file(base, prefix, suffix, data):
    import pyqrcodeng
    try:
        if data:
            qr = pyqrcodeng.create(data)
            filename = f"{prefix}_{suffix}.png"
            pngfile = os.path.join(base, filename)
            qr.png(pngfile, scale=5)
    except Exception as e:
        print(
            f"ERROR: unable to save QRCode data for {prefix} to {base} directory. Please fix problem and try again.")

def format_date(dt):
    try:
        arrow_date = arrow.get(dt)
        # result = arrow_date.humanize(granularity=["day", "hour", "minute"])
        result = arrow_date.humanize()
        return result
    except:
        return dt

#
# This is used to get a secret out of secretsmanager.
#
def get_secret(config, name):
    profile = config.get("aws_profile", "default")
    os.environ['AWS_PROFILE'] = profile
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=config.get("region", None)
    )

    # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
    # See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
    # We rethrow the exception by default.

    try:
        secret_value = client.get_secret_value(
            SecretId=name
        )
    except ClientError as e:
        print(f"Got exception: {str(e)}")

        if e.response['Error']['Code'] == 'DecryptionFailureException':
            # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InternalServiceErrorException':
            # An error occurred on the server side.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidParameterException':
            # You provided an invalid value for a parameter.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidRequestException':
            # You provided a parameter value that is not valid for the current state of the resource.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'ResourceNotFoundException':
            # We can't find the resource that you asked for.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
    else:
        # Decrypts secret using the associated KMS CMK.
        # Depending on whether the secret is a string or binary, one of these fields will be populated.
        if 'SecretString' in secret_value:
            secret = secret_value["SecretString"]
            return json.loads(secret)
        else:
            return None

#
# This is used to subscribe to an MQTT topic. Note that it isn't at all re-entrant, since it
# is meant to be used by a single command at a time. If you need to monitor multiple topics,
# open a new window and start a new shell process.
#
mqtt_client = None
on_data_callback = None
on_error_callback = None
show_raw = False
stop_after_one = False
mqtt_topic = None


def get_device_cert_path(team, project, model, serial, suffix):
    try:
        device_dir = get_iot_device_dir(team, project, model, serial)

        filename = f"{serial}_{suffix}.pem"
        filepath = os.path.join(device_dir, filename)
        return filepath

    except Exception as e:
        print(
            f"ERROR: unable to read device data for {serial} to {base} directory. Please delete device, fix problem, and try again")


def _get_monitor_cert_paths(team):
    ca_file_path = None
    cert_file_path = None
    public_key_file_path = None
    private_key_file_path = None

    try:
        config = load_config(team)
        if config:
            iot_monitor_rootca_filename = config.get("iot_monitor_rootca_filename", None)
            iot_monitor_cert_filename = config.get("iot_monitor_cert_filename", None)
            iot_monitor_public_key_filename = config.get("iot_monitor_public_key_filename", None)
            iot_monitor_private_key_filename = config.get("iot_monitor_private_key_filename", None)

            cert_path = path_for_certs(team)

            ca_file_path = os.path.join(cert_path, iot_monitor_rootca_filename)
            cert_file_path = os.path.join(cert_path, iot_monitor_cert_filename)
            public_key_file_path = os.path.join(cert_path, iot_monitor_public_key_filename)
            private_key_file_path = os.path.join(cert_path, iot_monitor_private_key_filename)
        else:
            print(f"ERROR: failed to load config data for team [ {team} ]")

        return ca_file_path, cert_file_path, public_key_file_path, private_key_file_path

    except Exception as e:
        print(
            f"ERROR: unable to read device data for {serial} to {base} directory. Please delete device, fix problem, and try again")


def _mqtt_callback(client=None, userdata=None, message=None):
    global mqtt_client, on_data_callback, on_error_callback, show_raw, stop_after_one
    try:
        if message:
            topic = message.topic
            payload_str = message.payload
            payload = json.loads(payload_str)
            if on_data_callback:
                result = on_data_callback(topic, payload, show_raw, stop_after_one)
                if not result:
                    #
                    # This kills the whole process group. A regular sys.exit(0) just kills the single thread.
                    #
                    os.kill(os.getpid(), signal.SIGINT)
                    #
                    # For Windows, we may want to call os._exit()
    except Exception as e:
        if on_error_callback:
            on_error_callback(str(e))


def _mqtt_onconnect(client=None, userdata=None):
    global mqtt_client, mqtt_topic

    # print(f"Connected. Subscribing {mqtt_client} to topic: {mqtt_topic}")
    mqtt_client.subscribe(mqtt_topic, 1, _mqtt_callback)
    print(f"Ready.")


def subscribe_to_mqtt_topic(team, project, model, serial, topic,
                            raw=False, stop=False,
                            on_data=None, on_error=None):
    global mqtt_client, on_data_callback, on_error_callback, show_raw, stop_after_one, mqtt_topic

    show_raw = raw
    stop_after_one = stop
    mqtt_topic = topic

    config = load_config(team)
    mqtt_port = config.get("mqtt_port", 8883)
    iot_endpoint = config.get("iot_endpoint", None)
    if not iot_endpoint:
        print(f"ERROR: iot_endpoint not defined. Something may have gone wrong during installation.")
        exit(1)


    # Now we look for the certs for this specific device. If not found, we use the
    # Monitor certs installed at time of
    ca_file_path = get_device_cert_path(team, project, model, serial, "rootca")
    private_key_file_path = get_device_cert_path(team, project, model, serial, "private")
    cert_file_path = get_device_cert_path(team, project, model, serial, "cert")

    if not (ca_file_path or private_key_file_path or cert_file_path):
        ca_file_path, cert_file_path, public_key_file_path, private_key_file_path = _get_monitor_cert_paths(team)

    # let's make a random client ID
    #
    alphabet = string.ascii_lowercase + string.digits
    uuid_str = ''.join(random.choices(alphabet, k=8))
    client_id = f"iot-cli-{uuid_str}"

    if not mqtt_client:
        mqtt_client = AWSIoTMQTTClient(client_id)
        mqtt_client.configureEndpoint(iot_endpoint, mqtt_port)
        mqtt_client.configureCredentials(ca_file_path, private_key_file_path, cert_file_path)
        mqtt_client.configureAutoReconnectBackoffTime(1, 32, 20)
        mqtt_client.configureOfflinePublishQueueing(-1)  # Infinite offline Publish queueing
        mqtt_client.configureDrainingFrequency(2)  # Draining: 2 Hz
        mqtt_client.configureConnectDisconnectTimeout(10)  # 10 sec
        mqtt_client.configureMQTTOperationTimeout(30)  # 30 sec
        # This is our private internal callback. We use it to watch for data, then process the
        # payload before sending it back up.
        #
        mqtt_client.onMessage = _mqtt_callback
        # mqtt_client.onOnline = _mqtt_onconnect
        on_data_callback = on_data        # we save the callback handlers passed down to us
        on_error_callback = on_error

        # Now we go connect. When actually connected, the onOnline handler is called.
        # There we can subscribe to the topic we're interested in.
        #
        mqtt_client.connectAsync(ackCallback=my_connack_callback)
        loopCount = 0
        while True:
            loopCount += 1
            time.sleep(1)

def my_connack_callback(mid, data):
    print(f"Connected to AWS IOT")
    _mqtt_onconnect(None, None)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.cli.auth import auth
from simpleiot.cli.cloud import cloud
from simpleiot.cli.data import data
from simpleiot.cli.datatype import datatype
from simpleiot.cli.device import device
from simpleiot.cli.firmware import firmware
from simpleiot.cli.location import location
from simpleiot.cli.model import model
from simpleiot.cli.project import project
from simpleiot.cli.team import team
from simpleiot.cli.template import template
from simpleiot.cli.toolchain import toolchain
from simpleiot.cli.twin import twin
from simpleiot.cli.update import update

@click.group()
def iotcli():
    pass

iotcli.add_command(auth)
iotcli.add_command(cloud)
iotcli.add_command(data)
iotcli.add_command(datatype)
iotcli.add_command(device)
iotcli.add_command(firmware)
iotcli.add_command(location)
iotcli.add_command(model)
iotcli.add_command(project)
iotcli.add_command(team)
iotcli.add_command(template)
iotcli.add_command(toolchain)
iotcli.add_command(twin)
iotcli.add_command(update)


if __name__ == '__main__':
    iotcli()



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table

console = Console()

@click.group()
def template():
    """Project Template management"""


@template.command()
@common_cli_params
@click.option('--name', help='Template name', required=True)
@click.option('--desc', help='Template Description', default="")
@click.option('--type', help='Template Type', default="project")
@click.option('--icon', '-icon_url', help='Icon URL', default="")
@click.option('--author', help='Template author', default="")
@click.option('--email', help='Template author email', default="")
@click.option('--dev_url', help='Template developer URL', default="")
@click.option('--license', help='Template license', default="")
@click.option('--zip_url', '-zip', help='Template zip', default="")
@click.option('--file', '-f', help='Template value file', type=click.Path(exists=True))
def add(team, profile, name, desc, type, icon, author, email, dev_url, license, zip_url, file):
    """
    Define a new Project template
    \f
    Example:

    $ iot template add --name "..."
    """
    try:
        config = preload_config(team, profile)
        payload = {
            "name": name,
            "desc": desc
        }
        if type:
            payload["type"] = type
        if icon:
            payload["icon_url"] = icon
        if author:
            payload["author"] = author
        if email:
            payload["email"] = email
        if dev_url:
            payload["dev_url"] = dev_url
        if license:
            payload["license"] = license
        if zip_url:
            payload["zip_url"] = zip_url
        if file:
            with open(file, "r") as f:
                value = f.readlines()
            if value:
                payload['value'] = value

        response = make_api_request('POST', config, 'template', json=payload)
        if response:
            data = response.json()
            #
            # This creates the right local directory, if needed
            #
            table = Table(show_header=True, header_style="bold green")
            table.add_column("ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            template_id = data.get("id", "")
            status = data.get("status", "ok")
            message = data.get("message", "")
            table.add_row(template_id, status, message)
            console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@template.command()
@common_cli_params
@click.option('--name', '--template', help='Template name', default=None)
@click.option('--id', help='Template ID', default=None)
@click.option('--full/--no-full', help='List Full Data', default=False)
def list(team, profile, name, id, full):
    """
    List defined Project templates
    \f
    Examples:

    $ iot template list --name "..."
    """
    try:
        config = preload_config(team, profile)
        multi = False

        if name:
            response = make_api_request('GET', config, f"template?name={name}")
        elif id:
            response = make_api_request('GET', config, f"template?id={id}")
        else:
            multi = True
            response = make_api_request('GET', config, f"template?all=true")

        data = response.json()

        if response:
            if full and not multi:
                show_detail(console, "Template", data)
                return

            table = Table(show_header=True, header_style="bold green")
            table.add_column("Template ID", style="dim", overflow="flow")
            table.add_column("Name")
            table.add_column("Date Created", justify="right")
            #print(f"RESULT: Type: {type(data)} - {data}")
            if multi:
                for d in data:
                    table.add_row(d['id'], d['name'], format_date(d['date_created']))
            else:
                table.add_row(data['id'], data['name'], format_date(data['date_created']))
        else:
            status = data.get('status', "")
            message = data.get('message', "")
            table = Table(show_header=True, header_style="red")
            table.add_column("Template List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@template.command()
@common_cli_params
@click.option('--id', help='Template ID', default=None)
@click.option('--template_name', help='Template name', default=None)
@click.option('--name', help='New Template name', default=None)
@click.option('--desc', help='Template Deescription', default=None)
@click.option('--type', help='Template Type', default="project")
@click.option('--icon', '-icon_url', help='Icon URL', default="")
@click.option('--author', help='Template author', default="")
@click.option('--email', help='Template author email', default="")
@click.option('--dev_url', help='Template developer URL', default="")
@click.option('--license', help='Template license', default="")
@click.option('--zip_url', '-zip', help='Template zip', default="")
def update(team, profile, id, template_name, name, desc, type, icon, author, email, dev_url,
           license, zip_url):
    """
    Update Project Template attributes
    \f
    Updates information on templates already in system. Note: the value
    can not be modified (since it's a big JSON). To edit that, delete the old
    one and create a new one.

    Examples:
    \b
    $ iot template update --name "Template Name" --desc "..."
    """
    try:
        config = preload_config(team, profile)

        params = {}

        if name:
            params["name"] = name
        if id:
            params["id"] = id
        if template_name:
            params["new_name"] = name
        if desc:
            params["desc"] = desc
        if type:
            params["type"] = type
        if icon:
            params["icon_url"] = icon
        if author:
            params["author"] = author
        if email:
            params["email"] = email
        if dev_url:
            params["dev_url"] = dev_url
        if license:
            params["license"] = license
        if zip_url:
            params["zip_url"] = zip_url

        url_params = urllib.parse.urlencode(params)
        url = f"template?{url_params}"
        print(f"Update Template URL: {url}")
        response = make_api_request('PUT', config, url)
        print(f"Response: {response}")
    except Exception as e:
        print(f"ERROR: {str(e)}")


@template.command()
@common_cli_params
@click.option('--name', help='Template name', default=None)
@click.option('--id', help='Template ID', default=None)
def delete(team, profile, name, id):
    """
    Delete an existing Project Template
    \f
    Examples:
    \b
    $ iot template delete --name "..."
    """
    try:
        config = preload_config(team, profile)
        response = None

        if name:
            print(f"Deleting Template with name: '{name}'")
            response = make_api_request('DELETE', config, f"template?name={name}")
        elif id:
            print(f"Deleting Template with ID: '{id}'")
            response = make_api_request('DELETE', config, f"template?id={id}")

        if response.status_code == 418:
            print(f"Invalid parameter. Record not found.")
            data = response.json()
            template_id = data.get('id', "")
            status = data.get('status', "")
            message = data.get('message', "")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Delete ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(template_id, status, message)
            console.print(table)

        elif response.status_code == 200:
            data = response.json()
            template_id = data.get('id', "")
            status = data.get('status', "")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_row(template_id, status)
            console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")



# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This is the base class for toolchain installer/uninstallers. Derived classes
# will then provide ways to install toolchains in a way that can be invoked by
# the firmware to compile and flash a device.
#
# We could have implemented this as a config file, but some toolchains require
# checking for conditions, so having it be done in code would allow inclusion
# of code to make sure everything is set up as needed.
#
from enum import Enum
from shutil import which
import os
import subprocess
import binascii
from pathlib import Path


class ToolchainLocation(Enum):
    LOCAL = 'local'                        # In local filesystem
    CONTAINER = 'container'                # Locally via Docker container
    CLOUD_CONTAINER = 'cloud_container'    # Remote container hosting, connection via API
    CLOUD_SERVICE = 'cloud_service'        # Remote central cloud service, connection via API

    def __getitem__(self, item):
        if isinstance(item, str):
            item = item.lower()
        return super().__getitem__(item)

#####################################################################

class ToolChainBase():
    def __init__(self,
                 alias,
                 name,
                 desc,
                 manufacturer,
                 processor,
                 opsys,
                 version="1.0.0",
                 can_compile=True,
                 can_flash=True,
                 can_compile_and_flash=True,
                 location=ToolchainLocation.LOCAL):
        self.alias = alias
        self.name = name
        self.desc = desc
        self.manufacturer = manufacturer
        self.processor = processor
        self.opsys = opsys
        self.can_compile = can_compile
        self.can_flash = can_flash
        self.can_compile_and_flash = can_compile_and_flash
        self.version = version
        self.location = location
        self.key = self._make_key(manufacturer, processor, opsys, version)
        self.alias_dict = {
            "man": manufacturer,
            "pro": processor,
            "ops": opsys,
            "ver": version,
            "loc": location
        }
        self.executable = ""


    def _make_key(self, manufacturer, processor, opsys, version):
        strkey = f"{manufacturer}::{processor}::{opsys}::{version}"
        hexbin = binascii.hexlify(strkey.encode("utf-8"))
        hexstr = str(hexbin, "utf-8")
        return hexstr

    def _app_exists(self, name):
        """Check whether name is on PATH and marked as executable."""
        return which(name) is not None

    def _file_exists(self, name):
        result = False
        if name:
            expanded_path = os.path.expanduser(name)
            result = os.path.exists(expanded_path)
        return result

    def _exec(self, command):
        x = subprocess.run(command, shell=True, capture_output=True)
        return x
        # return os.system(command)

    def _invoke_unbuffered(self, command):
        try:
            if sys.platform == "win32":
                # For windows we use wexpect
                import msvcrt
                msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
                os.system(command)

            else:
                # For mac or Linux we use pexpect
                import pexpect
                c = pexpect.spawnu(command)
                c.interact()
                c.kill(1)
        except Exception as e:
            pass

    def install_path(self, base):
        if type(base) != Path:
            base = Path(base)
        install_path = Path(os.path.expanduser(base)) / self.key
        if not install_path.exists():
            os.makedirs(install_path)
        return install_path

    def exec_path(self, install_path):
        if type(install_path) != Path:
            install_path = Path(install_path)

        exec_path = install_path / self.executable
        return exec_path

    def get_installed_toolchain_version(self, install_path=None):
        """ Return the version of the installed tool """
        raise NotImplementedError


    # The following may be overridden for each platform.
    #
    def install_windows(self, base, location):
        """ Install this toolchain """
        raise NotImplementedError

    def install_mac(self, base, location):
        """ Install this toolchain """
        raise NotImplementedError

    def install_unix(self, base, location):
        """ Install this toolchain """
        raise NotImplementedError

    def uninstall_windows(self, base, location):
        """ Uninstall this toolchain """
        raise NotImplementedError

    def uninstall_mac(self, base, location):
        """ Uninstall this toolchain """
        raise NotImplementedError

    def uninstall_unix(self, base, location):
        """ Uninstall this toolchain """
        raise NotImplementedError

    def reset_windows(self, base, location):
        """ Reset this toolchain to its original factory settings. """
        raise NotImplementedError

    def reset_mac(self, base, location):
        """ Reset this toolchain to its original factory settings. """
        raise NotImplementedError

    def reset_unix(self, base, location):
        """ Reset this toolchain to its original factory settings. """
        raise NotImplementedError

    #
    # NOTE: this could be expanded to support additional build actions, etc.
    #
    def build(self, base, dirpath, command):
        """ User the tool spec to compile/link the code into binaries """
        raise NotImplementedError

    def flash(self, base, dirpath, command):
        """ User the tool spec to flash the code into binaries """
        raise NotImplementedError

    def build_and_flash(self, base, dirpath, command):
        """ User the tool spec to build and flash the source into binaries """
        raise NotImplementedError



# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# These are common configuration management routines used by subsystems that need
# access to configuration data.
#
# Note that a super-set of this is in the iotcli folder so it can be run standalone
# with a minimal version of the config.json file for those who don't need to
# download the whole package. Also, note that this used to be a symbolic link to that
# file, but after the refactoring, we copied it from there so there would be no
# external dependencies to the installer, but the two files have diverged since.
# If changes are made to that file, this one has to be manually merged.
#
# The other config file is at: simpleiot/sources/iotapi/util/config.py
#
# The configuration system for SimpleIOT is based on files placed in the following
# hierarchy:
#
# ~ (home directory)
#  .simpleiot (root of SimmpleIOT)
#     {team-name}
#        bootstrap.json - json file generated by the bootstrap process
#        cdkoutput.json - json file output from running the CDK
#        config.json - merged JSON file input by other subsystems
#        projects
#           {project-name}
#           ...
#                models
#                   {model-name}
#                   ...
#                       {devices}
#                          {device-1-serial}
#                          {device-2}
#                              {serial}-cert.pem
#                              {serial}-private-key.pem
#                              {serial}-public-key.pem
#
import re
import shutil
import stat
import os
import json
import tempfile
import boto3
from botocore.exceptions import ClientError
import questionary
import keyring
import webbrowser
import traceback
from questionary import Validator, ValidationError, prompt
import click
import functools


SIMPLEIOT_LOCAL_ROOT = "~/.simpleiot"
DEFAULT_TEAM = "simpleiot"


# This just makes sure the directory structure is set up properly, and has proper access
# privileges. The 'sync' mechanism re-creates all the certs with the devices and models.
#
# NOTE also that the serial numbers MUST be normalized so they can be supported by the
# local filesystem.
#
# NOTE: if the user has specified a different project root, we still look for it in the
# config.json file in ~/.iot.
#

class Config(object):
    def __init__(self, team, use_sso, profile, account, region, identity, userpool,
                 client_id, api_endpoint, config, debug):
        self.team = team
        self.use_sso = use_sso
        self.profile = profile
        self.account = account
        self.region = region
        self.identity = identity
        self.userpool = userpool
        self.client_id = client_id
        self.config = config
        self.debug = debug
        self.api_endpoint = api_endpoint

        # We get any stored API tokens from the keyring
        # If not defined, we'll have to signal that they have to first do an
        # 'iot auth login --usenname .... --password ...' to get a token.
        #
        # If in SSO mode, we won't be having any such token.
        #
        self.token = get_stored_api_token(self)
        # if not self.token:
        #     print(f"WARNING: missing authentication. Make sure you run 'iot auth login ...' before proceeding")

def preload_config(team,
              profile=None,
              debug=False):

    config = None
    try:
        # config_file = None
        # conf_dir = get_conf_dir()
        # if not conf_dir:
        #     if profile == None:
        #         config_path = os.path.join(conf_dir, "config.json")
        #     else:
        #         config_path = os.path.join(conf_dir, f"config-{profile}.json")
        #
        # with open(config_path, "r") as infile:
        #     config = json.load(infile)

        config_data = load_config(team)
        if config_data:
            # NOTE: the camelcase parameters are generated by CDK/CloudFormation.
            # CloudFormation does not allow underlines in output names.
            #
            team = config_data.get("team", team)
            account = config_data.get("account", None)
            region = config_data.get("region", None)
            api_endpoint = config_data.get("apiEndpoint", None)
            identity = config_data.get("cognitoIdentityPoolId", None)
            client_id = config_data.get("cognitoClientId", None)

            use_sso = config_data.get("use_sso", False)

            if use_sso:
                profile = None
                userpool = None
            else:
                profile = config_data.get("aws_profile", None)
                userpool = config_data.get("cognitoUserPoolId", None)

            if debug:
                if team:
                    print(f"Team ID: {team}")
                if profile:
                    print(f"Profile: {profile}")
                if account:
                    print(f"Account: {account}")
                if region:
                    print(f"Region: {region}")
                if identity:
                    print(f"Cognito Identity Pool: {identity}")
                if userpool:
                    print(f"Cognito User Pool: {userpool}")
                if client_id:
                    print(f"Cognito Client ID: {client_id}")
                if api_endpoint:
                    print(f"Endpoint: {api_endpoint}")

            config = Config(team, use_sso, profile, account, region, identity,
                            userpool, client_id, api_endpoint, config, debug)
        return config
    except Exception as e:
        print(f"ERROR: could not open configuration file for profile in ~/.simpleiot. {str(e)}")
        exit(1)



# This is a common wrapper to be used by all commands to add common parameters. Note that
# adding these means the options MUST be on the function definition.

def common_cli_params(func):
    @click.option("--team", "-t", envvar="IOT_TEAM", help="IOT Team ID", required=True)
    @click.option("--profile", "-pr", help="AWS Profile", envvar="AWS_PROFILE", default=None)
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


def _normalize_path_name(src):
    """
    Normalize a path name by removing all characters that aren't in a sanitized set.
    :param src: source name
    :return: cleaned path name
    """
    clean = re.sub('[^a-zA-Z0-9_-]', '', src)
    return clean


def list_all_teams():
    """
    Return a list of all teams. These are identified as directories inside the ~/.simpleiot
    directory that contain a config.json file. We ignore directories that start with an underline.
    These are for internal use.
    """
    result = {}
    root_iot_dir = os.path.expanduser(SIMPLEIOT_LOCAL_ROOT)
    subfolders = [f.name for f in os.scandir(root_iot_dir) if f.is_dir() and f.name[0] != "_"]

    for team_name in subfolders:
        try:
            config = load_config(team_name, exit_on_error=False)
            if config:
                result[team_name] = config
        except Exception as e:
            pass

    return result

def get_iot_team_dir(team=DEFAULT_TEAM, create=True):
    """
    At the root of the IOT directory, we create directories for each
    given team. Inside each one will be the subdirectories for projects,
    models, etc. The team name will be 'normalized' by taking out
    invalid characters.

    Note that this may cause collisions if someone creates team names
    that differ only with invalid characters, for example ABC$ and ABC!
    will both be normalized to ABC.

    :param team: name of team.
    :param create: If set to true, creates the directory path if it doesn't exist
    :return:
    """
    result = None

    try:
        root_iot_dir = os.path.expanduser(SIMPLEIOT_LOCAL_ROOT)
        if not os.path.exists(root_iot_dir):
            if create:
                os.makedirs(root_iot_dir)

        normal_team = _normalize_path_name(team)
        team_dir = os.path.join(root_iot_dir, normal_team)
        if not os.path.exists(team_dir):
            if create:
                os.makedirs(team_dir)

        result = team_dir
    except Exception as e:
        print("ERROR creating IOT root directory: " + str(e))

    return result


def get_iot_project_dir(team, project, create=True):
    """
    This returns the root of a single project directory.

    :param team: Team name
    :param project: Project name
    :param create: If set to true, creates the directory path if it doesn't exist
    :return:
    """
    result = None
    team_root = get_iot_team_dir(team, create)
    if team_root:
        normal_project = _normalize_path_name(project)
        project_dir = os.path.join(team_root, "projects", normal_project)
        if create:
            if not os.path.exists(project_dir):
                os.makedirs(project_dir)

        result = project_dir
    return result


def get_iot_model_dir(team, project, model, create=True):
    """
    This returns the root directory where model information is saved.

    :param team: Team name
    :param project: Project name
    :param model: Model name
    :param create: If set to true, creates the directory path if it doesn't exist
    :return:
    """
    result = None
    project_dir = get_iot_project_dir(team, project, create)
    if project_dir:
        normal_model = _normalize_path_name(model)
        model_dir = os.path.join(project_dir, "models", normal_model)
        if create:
            if not os.path.exists(model_dir):
                os.makedirs(model_dir)

        result = model_dir
    return result


def get_iot_device_dir(team, project, model, device, create=True):
    """
    This returns the root directory where individual device information is saved.

    :param team: Team name
    :param project: Project name
    :param model: Model name
    :param device: Device serial ID
    :param create: If set to true, creates the directory path if it doesn't exist
    :return:
    """
    result = None
    model_dir = get_iot_model_dir(team, project, model, create)
    if model_dir:
        normal_device = _normalize_path_name(device)
        device_dir = os.path.join(model_dir, "devices", normal_device)
        if create:
            if not os.path.exists(device_dir):
                os.makedirs(device_dir)

        result = device_dir
    return result


def delete_even_readonly_files(action, name, exc):
    """
    Utility to force a file to writeable and then remove it.
    This is passed as a shutil.rmtree error handler to clean up a file
    before deletion.

    :param action:
    :param name:
    :param exc:
    :return:
    """
    try:
        os.chmod(name, stat.S_IWRITE)
        os.remove(name)
    except Exception:
        pass


def delete_iot_device_dir(team, project, model, device):
    """
    Delete all the on-disk cache data having to do with a single Device.

    :param team:
    :param project:
    :param model:
    :param device:
    :return:
    """
    device_dir = get_iot_device_dir(team, project, model, device, create=False)
    #print(f"GOT device directory: {device_dir}")
    if device_dir:
        shutil.rmtree(device_dir, onerror=delete_even_readonly_files)


def delete_iot_model_dir(team, project, model):
    """
    Delete all the on-disk cache data having to do with a single Model.

    :param team:
    :param project:
    :param model:
    :return:
    """
    model_dir = get_iot_model_dir(team, project, model, create=False)
    if model_dir:
        shutil.rmtree(model_dir, onerror=delete_even_readonly_files)


def delete_iot_project_dir(team, project):
    """
    Delete all the on-disk cache data having to do with a single Project.

    :param team:
    :param project:
    :return:
    """
    project_dir = get_iot_project_dir(team, project, create=False)
    if project_dir:
        shutil.rmtree(project_dir, onerror=delete_even_readonly_files)


def delete_iot_team_dir(team):
    """
    Delete all the on-disk cache data having to do with a single Team.

    :param team:
    :return:
    """
    team_dir = get_iot_team_dir(team, create=False)
    if team_dir:
        shutil.rmtree(team_dir, onerror=delete_even_readonly_files)

#################################
# Configuration loading routines
#
# These are used to centrally load config files for each Team
#
def path_for_bootstrap_file(team=DEFAULT_TEAM):
    team_path = get_iot_team_dir(team)
    bootstrap_path = os.path.join(team_path, "bootstrap.json")
    return bootstrap_path


def path_for_cdkoutput_file(team=DEFAULT_TEAM):
    team_path = get_iot_team_dir(team)
    cdkoutput_path = os.path.join(team_path, "cdkoutput.json")
    return cdkoutput_path

# A config file is a merged JSON file that combines boothstrap and cdkoutput
# data.

def path_for_config_file(team=DEFAULT_TEAM):
    team_path = get_iot_team_dir(team)
    config_path = os.path.join(team_path, "config.json")
    return config_path


def path_for_certs(team=DEFAULT_TEAM):
    team_path = get_iot_team_dir(team)
    certs_path = os.path.join(team_path, "certs")
    if not os.path.exists(certs_path):
        os.makedirs(certs_path)
    result = certs_path
    return result


def load_bootstrap_config(team=DEFAULT_TEAM):
    config_data = None
    bootstrap_path = None
    try:
        bootstrap_path = path_for_bootstrap_file(team)

        with open(bootstrap_path, "r") as infile:
            config_data = json.load(infile)
    except Exception as e:
        print(f"ERROR loading bootstrap config file: [{bootstrap_path}]: {str(e)}")
        exit(1)

    return config_data


def load_defaults_config(defaults_path):
    config_data = None
    try:
        with open(defaults_path, "r") as infile:
            config_data = json.load(infile)
    except Exception as e:
        print(f"ERROR loading defaults config file: [{defaults_path}]: {str(e)}")
        exit(1)

    return config_data


def load_cdkoutput_config(team=DEFAULT_TEAM):
    config_data = None
    cdkoutput_path = None
    try:
        cdkoutput_path = path_for_cdkoutput_file(team)

        with open(cdkoutput_path, "r") as infile:
            config_root = json.load(infile)
            config_data = config_root.get("Iotcdk", None)
    except Exception as e:
        print(f"ERROR loading CDK output config file: [{cdkoutput_path}]: {str(e)}")
        exit(1)

    return config_data


def does_team_exist(team=DEFAULT_TEAM):
    result = False
    config_data = None
    config_path = None
    try:
        config_path = path_for_config_file(team)

        with open(config_path, "r") as infile:
            config_data = json.load(infile)

        account = config_data.get("account")
        result = True

    except Exception as e:
        pass

    return result


def load_config(team=DEFAULT_TEAM, exit_on_error=True):
    config_data = None
    config_path = None
    try:
        config_path = path_for_config_file(team)
        if not os.path.exists(config_path):
            if exit_on_error:
                print(f"ERROR: team '{team}' not recognized.")
                exit(1)
            else:
                return config_data # None

        with open(config_path, "r") as infile:
            config_data = json.load(infile)
    except Exception as e:
        print(f"ERROR: could not locate configuration data for project [{team}].")

    return config_data


####################################################################################
# Temp files - these are used to pass data between various phases of installation.
#
def save_to_tempfile(filename, data):
    try:
        tempdir = tempfile.gettempdir()
        outfile = os.path.join(tempdir, filename)
        with open(outfile, 'w') as out:
            out.write(data + '\n')
    except Exception as e:
        print(f"ERROR: could not save data to tempfile: {str(e)}")
        raise e


def load_from_tempfile(filename):
    try:
        tempdir = tempfile.gettempdir()
        infile = os.path.join(tempdir, filename)
        with open(infile, 'r') as out:
            result = out.read().replace('\n','')
            return result
    except Exception as e:
        print(f"ERROR: could not read data from tempfile {filename}")
        raise e


def load_from_tempfile_and_delete(filename):
    try:
        tempdir = tempfile.gettempdir()
        infile = os.path.join(tempdir, filename)
        with open(infile, 'r') as out:
            result = out.read().replace('\n','')
        os.remove(infile)
        return result
    except Exception as e:
        print(f"ERROR: could not read data from tempfile {filename}")
        raise e


#######

def ask_to_confirm_delete():
    is_delete = questionary.text("\nAre you sure you want to do this. Enter 'DELETE' to confirm:").ask()
    if is_delete == 'DELETE':
        return True
    else:
        return False


def ask_to_confirm_yesno(prompt):
    response = False
    confirm = questionary.text(f"{prompt} [N/y]:").ask()
    if confirm:
        confirm_l = confirm.lower()
        if confirm_l == 'y' or confirm_l == "ye" or confirm_l == "yes":
           response = True

    return response

#########

#
# Username/password/API token/SSO session management
#
# NOTE: a service name is synthesized with the team name. If not specified
# in the config file, we use default. This way we can store tokens and passwords
# for multiple teams.
#
def _get_service(config):
    team = config.team
    if not team:
        team = DEFAULT_TEAM

    service = f"simpleiot_{team}"
    return service

def get_stored_key(config, key):
    service = _get_service(config)
    value = keyring.get_password(service, key)
    return value

def set_stored_key(config, key, value):
    service = _get_service(config)
    keyring.set_password(service, key, value)

def get_stored_username(config):
    return get_stored_key(config, "username")

def get_stored_password(config):
    return get_stored_key(config, "password")

def get_stored_api_token(config):
    return get_stored_key(config, "api_token")

def get_stored_access_key(config):
    return get_stored_key(config, "access_key")

def get_stored_access_secret(config):
    return get_stored_key(config, "access_secret")

def get_stored_session_token(config):
    return get_stored_key(config, "session_token")

def store_username(config, username):
    set_stored_key(config, "username", username)

def store_password(config, password):
    set_stored_key(config, "password", password)

def store_api_token(config, token):
    set_stored_key(config, "api_token", token)

def store_access_key(config, key):
    set_stored_key(config, "access_key", key)

def store_access_secret(config, secret):
    set_stored_key(config, "access_secret", secret)

def store_session_token(config, token):
    set_stored_key(config, "session_token", token)

def clear_api_token(config):
    try:
        service = _get_service(config)
        keyring.delete_password(service, "api_token")
    except Exception as e:
        pass

def clear_sso_tokens(config):
    try:
        service = _get_service(config)
        keyring.delete_password(service, "access_key")
        keyring.delete_password(service, "access_secret")
        keyring.delete_password(service, "session_token")
    except Exception as e:
        pass

def clear_all_auth(config):
    try:
        service = _get_service(config)
        keyring.delete_password(service, "username")
        keyring.delete_password(service, "password")
        keyring.delete_password(service, "api_token")
        keyring.delete_password(service, "access_key")
        keyring.delete_password(service, "access_secret")
        keyring.delete_password(service, "session_token")
    except Exception as e:
        pass

################
# SSO Login

class IsEmptyValidator(Validator):
    def validate(self, document):
        if len(document.text) == 0:
            raise ValidationError(
                message="Please enter a value",
                cursor_position=len(document.text),
            )


#
# This function tries to login using SSO credentials. If SSO credentials were saved from the last time
# they're used here. If not, we ask for them.
#
def login_with_sso(config):
    try:
        GRANT_TYPE = 'urn:ietf:params:oauth:grant-type:device_code'
        sso_url_stored = config.get("sso_url", "")
        url_ok = False
        import validators

        while not url_ok:
            sso_url = questionary.text("AWS SSO url?", validate=IsEmptyValidator, default=sso_url_stored).ask()
            url_ok = validators.url(sso_url)
            if not url_ok:
                print("ERROR: Invalid URL")

        sso_region_stored = config.get("region", None)
        session = boto3.session.Session()
        regions = session.get_available_regions('sso-oidc')
        if len(regions) == 0:
            print("ERROR: SSO not supported in these regions")
            exit(1)

        if not sso_region_stored:
            sso_region_stored = regions[0]

        sso_region = questionary.select("AWS SSO region: ", choices=regions, default=sso_region_stored).ask()

        sso_oidc = boto3.client('sso-oidc', region_name=sso_region)
        try:
            resp = sso_oidc.register_client(clientName="simpleiot", clientType="public")
            client_id = resp['clientId']
            client_secret = resp['clientSecret']

            resp = sso_oidc.start_device_authorization(
                clientId=client_id, clientSecret=client_secret, startUrl=sso_url)
            device_code = resp['deviceCode']
            signin_url = resp['verificationUriComplete']
            webbrowser.open(signin_url)

            access_token = False

            while not access_token:
                try:
                    resp = sso_oidc.create_token(clientId=client_id, clientSecret=client_secret,
                                                 grantType=GRANT_TYPE, deviceCode=device_code)
                    access_token = resp['accessToken']
                except sso_oidc.exceptions.AuthorizationPendingException:
                    continue
                except Exception as e:
                    print(f"ERROR: CREATE SSO TOKEN EXCEPTION: {str(e)}")
                    exit(1)

            sso = boto3.client('sso', region_name=sso_region)

            resp = sso.list_accounts(maxResults=1000, accessToken=access_token)
            account_array = resp['accountList']
            account_id = None
            account_name= None
            account_role = None

            if len(account_array) == 0:
                print("ERROR: No valid SSO accounts found")
                exit(1)

            # If we only have one account, we don't bother asking
            #
            if len(account_array) == 1:
                account = account_array[0]
                account_id = account['accountId']
                account_name = account['accountName']
            else:
                account_list = {}
                account_names = {}
                for account in account_array:
                    _name = account['accountName']
                    _email = account['emailAddress']
                    _id = account['accountId']
                    account_list[f"{_name} ({_email})"] = _id
                    account_names[_id] = _name

                account_names = list(account_list.keys())
                account_choice = questionary.select("Select SSO Account: ", choices=account_names).ask()
                account_id = account_list[account_choice]
                account_name = account_names[account_id]

            resp = sso.list_account_roles(maxResults=1000,
                                          accessToken=access_token,
                                          accountId=account_id)
            role_array = resp['roleList']
            if len(role_array) == 0:
                print("ERROR: no roles found for account. Please correct and re-run")
                exit(1)

            if len(role_array) == 1:
                account_role = role_array[0]['roleName']
            else:
                role_list = []
                for role in role_array:
                    role_list.append(role['roleName'])

                account_role = questionary.select("Select SSO Account Role: ", choices=role_list).ask()

            resp = sso.get_role_credentials(roleName=account_role, accountId=account_id,
                                            accessToken=access_token)
            access_key = resp['roleCredentials']['accessKeyId']
            access_secret = resp['roleCredentials']['secretAccessKey']
            session_token = resp['roleCredentials']['sessionToken']

            return sso_url, \
                   sso_region, \
                   account_id, \
                   account_role, \
                   account_name, \
                   access_key, \
                   access_secret, \
                   session_token

        except sso_oidc.exceptions.InvalidRequestException:
            print("ERROR: Invalid Request to SSO")
            exit(1)
        except sso_oidc.exceptions.TooManyRequestsException:
            print("ERROR: Too many requests Exception")
            exit(1)
        except sso_oidc.exceptions.UnauthorizedException:
            print("ERROR: Unauthorized Exception")
            exit(1)
        except sso_oidc.exceptions.ResourceNotFoundException:
            print("ERROR: Resource Not Found Exception")
            exit(1)
        except Exception as e:
            print(f"ERROR: {str(e)}")
            traceback.print_exc()
            return None

    except Exception as e:
        print(f"ERROR: {str(e)}")
        traceback.print_exc()
        return None

##############################################################################
# This allows looking up a string from elements of an Enum.
# Used to lookup device model fields. The lookup is case-insensitive.
#
def enum_from_str(en, nm):
    try:
        for item in en:
            if nm.lower() == item.name.lower():
                return item.value

        raise ValueError(f"Invalid value specified for enum {str(en)}")
    except Exception as e:
        print(f"Error: {e}")
        return None

#
# This reverses the above and returns the string value of an enum.
# Used to display or export an enum into a config file.
#
def enum_to_str(en, item):
    result = None
    try:
        result = en(item).name.lower()
    except Exception as e:
        pass

    return result

# This does the above, except it looks for a list of string values,
# which it then turns into an ORed bitflag.
#
# The values are assumed to be comma-separated and each a valid item in the
# enum. We don't check for duplicates.
#
def enum_from_str_list(en, nml):
    flag = 0
    slist = nml.split(",")
    try:
        for st in slist:
            for item in en:
                if st.strip().lower() == item.name.lower():
                    flag |= int(item.value)
                    break
        return flag
    except Exception as e:
        print(f"Error: {e}")
        return None

#
# This does the reverse, except it returns a list of enums.
#
def enum_to_str_list(en, value):
    result = []
    try:
        for b in range(1, len(en) + 1):
            cb = b - 1
            item = en(1 << cb)
            if (value & (1 << (b - 1))):
                result.append(item.name)
        return ", ".join(result)
    except Exception as e:
        print(f"ERROR: {str(e)}")
        pass
    return result

#
# This is used to get a secret out of secretsmanager.
#
def get_secret(config, name):
    profile = config.get("aws_profile", "default")
    os.environ['AWS_PROFILE'] = profile
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=config.get("region", None)
    )

    # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
    # See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
    # We rethrow the exception by default.

    try:
        secret_value = client.get_secret_value(
            SecretId=name
        )
    except ClientError as e:
        print(f"Got exception: {str(e)}")

        if e.response['Error']['Code'] == 'DecryptionFailureException':
            # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InternalServiceErrorException':
            # An error occurred on the server side.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidParameterException':
            # You provided an invalid value for a parameter.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidRequestException':
            # You provided a parameter value that is not valid for the current state of the resource.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'ResourceNotFoundException':
            # We can't find the resource that you asked for.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
    else:
        # Decrypts secret using the associated KMS CMK.
        # Depending on whether the secret is a string or binary, one of these fields will be populated.
        if 'SecretString' in secret_value:
            secret = secret_value["SecretString"]
            return json.loads(secret)
        else:
            return None



import serial.tools.list_ports

#device: /dev/cu.SLAB_USBtoUART
#name: cu.SLAB_USBtoUART
#description: CP2104 USB to UART Bridge Controller
#hwid: USB VID:PID=10C4:EA60 SER=02042965 LOCATION=2-4.1
#vid: 4292
#pid: 60000

FILTER_VID = [0x1a86, 0x10c4]
FILTER_PID = [0x55d4, 0xea60]


devices = []

for item in serial.tools.list_ports.comports():
  if item.vid in FILTER_VID and item.pid in FILTER_PID:
    devices.append(item)


for dev in devices:
  print(f"Found CP210 device: {dev.name} - at port: {dev.device}")




#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This command is used to load up the digital twin file and the datapoint
# values.
#
# Use this to upload the generated USDZ, GLB, or GLTB file up to S3, then
# mark its location into the model record.
#
# You can also use this to indicate the position for each datapoint on the
# 3D model.
#

import click
from simpleiot.common.utils import make_api_request, show_detail, format_date
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table
import boto3
from boto3.s3.transfer import S3Transfer
import os, sys, threading

console = Console()

#
# This class updates upload progress and displays it on the console.
#
class UploadProgress(object):
    def __init__(self, filename):
        self._filename = filename
        self._size = float(os.path.getsize(filename))
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = (self._seen_so_far / self._size) * 100
            sys.stdout.write(
                "\r%s  %s / %s  (%.2f%%)" % (
                    self._filename, self._seen_so_far, self._size,
                    percentage))
            sys.stdout.flush()


@click.group()
def twin():
    """Manage Digital 3D Twin files"""


@twin.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--file", "--input", help="USDZ/GLB/GLTF local file", required=True)
def file(team, profile, project, model, file):
    """
    Upload 3D Twin file to secure project bucket
    \f
    Examples:

    $ iot twin file --project="..." --model="..." --file="..."

    The file is uploaded to the S3 bucket created during installation for
    digital twin files.

    """
    # NOTE: we will have to  pass these values inside the 'team' file if we want to let someone
    # other than the admin upload models.
    #
    try:
        config = preload_config(team, profile)
        bucket = config.config.get("templateBucketName", None)
        uuid_suffix = config.config.get("uuidSuffix", "")
        region = config.config.get("region", "")
        aws_profile = config.config.get("aws_profile", None)

        gen_file_url = None

        if not bucket:
            print(f"ERROR: no model upload bucket name found. Perhaps the installer didn't finish properly? Skipping.")
            exit(1)

        base_file=os.path.basename(file)
        gen_upload_filename=f"twin/{uuid_suffix}_{base_file}"

        if not os.path.exists(file):
            print(f"ERROR: file {file} not found")
            exit(1)

        gen_file_abs = os.path.abspath(file)

        print(f"Uploading {file}...")
        if aws_profile:
            boto3.setup_default_session(profile_name=aws_profile)

        s3 = boto3.client('s3', region)
        transfer = S3Transfer(s3)
        upload_args = {"ContentType": "application/zip",
                       "ACL": "bucket-owner-full-control"}

        if transfer:
            try:
                transfer.upload_file(filename=gen_file_abs,
                                     bucket=bucket,
                                     key=gen_upload_filename,
                                     callback=UploadProgress(file),
                                     extra_args=upload_args)
                gen_file_url = f"{s3.meta.endpoint_url}/{bucket}/{gen_upload_filename}"
            except Exception as e:
                print(f"ERROR uploading to S3: {str(e)}. Skipping.")
                # traceback.print_exc()
                exit(1)

        if not gen_file_url:
            print(f"Could not upload file {file} to S3.")
            exit(1)

        payload = {
            "project_name": project,
            "model": model,
            "glb_url": gen_file_url
        }

        url_params = urllib.parse.urlencode(payload)
        url = f"model?{url_params}"
        response = make_api_request('PUT', config, url)
        data = response.json()

        if response:
            print(f"\n\nSuccess. Model URL set to: {gen_file_url}")
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("List Status")
            table.add_column("Message")
            table.add_row(status, message)
            console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@twin.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--type", "--name", help="Data Type name", required=True)
@click.option("--position", "--data_position", help="DataType Twin position")
@click.option("--normal", "--data_normal", help="DataType Twin normal")
def data(team, profile, project, model, type, position, normal):
    """
    Update position and normal data for Digital 3D Twin
    \f
    Updates the position and normal data for a datatype to show on a digital twin.
    """

    try:
        config = preload_config(team, profile)
        payload = {
            "project_name": project,
            "model": model
        }
        if type:
            payload["name"] = type
        if position:
            payload["data_position"] = position
        if normal:
            payload["data_normal"] = normal


        url_params = urllib.parse.urlencode(payload)
        url = f"datatype?{url_params}"
        response = make_api_request('PUT', config, url)
        data = response.json()

        if response:
            model_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("DataType Update ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(model_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")




# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# Toolchain installer classes
#
import platform
import binascii
import traceback
import sys
import os
import shutil
from pathlib import Path

DEFAULT_TOOLCHAIN_BASE="~/.simpleiot/_toolchain"


class Toolchain:
    toolchain_list = {}
    toolchain_alias = {}

    def __init__(self):
        # NOTE: we import the import of implementation classes here so we don't create import loops.
        # This way, the implementation classes can call class methods without causing import loops.
        #
        from .ToolchainESP32Arduino_1_0_0 import ToolchainESP32Arduino_1_0_0

        self._register(ToolchainESP32Arduino_1_0_0())

        # Add import/list of other toolchains here, for example:
        # self._register("microchip", "imx7", "freertos", "1.0.0", ToolchainIMX7FreeRTOS_1_0_0)

    @classmethod
    def base(cls):
        """
        Returns the default base directory for toolchains. On devices that use external installation
        tools (like Mac with Homebrew) this is ignored.
        """
        return DEFAULT_TOOLCHAIN_BASE

    def _make_key(self, manufacturer, processor, opsys, version):
        strkey = f"{manufacturer}::{processor}::{opsys}::{version}"
        hexbin = binascii.hexlify(strkey.encode("utf-8"))
        hexstr = str(hexbin, "utf-8")
        return hexstr

    # The key and alias_dict are calculated in the constructor
    #
    def _register(self, cls):
        self.toolchain_list[cls.key] = cls
        self.toolchain_alias[cls.alias] = cls.alias_dict

    def _resolve_alias(self, alias):
        cls = self.toolchain_alias.get(alias, None)
        if not cls:
            print(f"ERROR: Toolchain alias '{alias}' not recognized.")
            exit(1)
        return cls

    def _load_config(self):
        print("Load local config file and see what's installed on this machine")

    def _set_config(self, manufacturer, processor, opsys, version, location, binary):
        key = self._make_key(manufacturer, processor, opsys, version)
        print("Save configuration data for this platform -- overwrites old one")

    def _delete_config(self, manufacturer, processor, opsys, version, location, binary):
        key = self._make_key(manufacturer, processor, opsys, version)
        print("Deletes configuration data for this platform--after install")

    def install_alias(self, base, alias):
        params = self._resolve_alias(alias)
        self.install(base,
                     params["man"],
                     params["pro"],
                     params["ops"],
                     params["ver"],
                     params["loc"])


    def install(self, base, manufacturer, processor, opsys, version, location):
        try:
            key = self._make_key(manufacturer, processor, opsys, version)
            installer = self.toolchain_list.get(key, None)
            if installer:
                install_dir = installer.install_path(base)

                opsys = platform.system()
                if opsys == "Darwin":
                    installer.install_mac(install_dir, location)
                elif opsys == "Windows":
                    installer.install_windows(install_dir, location)
                elif opsys == "Linux":
                    installer.install_unix(install_dir, location)
                else:
                    print(f"ERROR: operating system not supported")
            else:
                print(f"ERROR: unable to install toolchain for specified device")
                exit(1)
        except Exception as e:
            print(f"ERROR installing tool: {str(e)}")
            print(traceback.format_exc())
            exit(1)


    def uninstall(self, base, manufacturer, processor, opsys, version, location):
        try:
            key = self._make_key(manufacturer, processor, opsys, version)
            installer = self.toolchain_list.get(key, None)
            if installer:
                install_dir = installer.install_path(base)

                opsys = platform.system()
                if opsys == "Darwin":
                    installer.uninstall_mac(install_dir, location)
                elif opsys == "Windows":
                    installer.uninstall_windows(install_dir, location)
                elif opsys == "Linux":
                    installer.uninstall_unix(install_dir, location)
                else:
                    print(f"ERROR: operating system not supported")

                # Also remove the directory in .simpleiot if it still exists
                #
                if os.path.exists(install_dir):
                    shutil.rmtree(install_dir)

                print("Done.")
            else:
                print(f"ERROR: unable to uninstall toolchain for specified device")
                exit(1)
        except Exception as e:
            print(f"ERROR uninstalling tool: {str(e)}")
            exit(1)

    def reset(self, base, manufacturer, processor, opsys, version, location):
        try:
            print("Loading defaults for toolchain...")
            key = self._make_key(manufacturer, processor, opsys, version)
            installer = self.toolchain_list.get(key, None)
            if installer:
                opsys = platform.system()
                if opsys == "Darwin":
                    installer.reset_mac(base, location)
                elif opsys == "Windows":
                    # For windows, we install the tool in a custom directory under ~/.simpleiot/_toolchain/{hash}
                    install_path = Path(base) / key
                    installer.reset_windows(install_path, location)
                elif opsys == "Linux":
                    installer.reset_unix(base, location)
                else:
                    print(f"ERROR: operating system not supported")
            else:
                print(f"ERROR: No toolchain found. Unable to reset toolchain for specified device")
                exit(1)
        except Exception as e:
            print(f"ERROR resetting tool to default settings: {str(e)}")
            exit(1)

    def build(self, base, manufacturer, processor, opsys, version, location, dirpath, command):
        try:
            key = self._make_key(manufacturer, processor, opsys, version)
            installer = self.toolchain_list.get(key, None)
            if installer:
                result = installer.build(base, dirpath, command)
                return result
            else:
                print(f"ERROR: unable to find toolchain for specified device")
                exit(1)
        except Exception as e:
            print(f"ERROR running tool: {str(e)}")
            print(traceback.format_exc())
            exit(1)

    def build_and_flash(self, base, manufacturer, processor, opsys, version, location, dirpath, command):
        try:
            key = self._make_key(manufacturer, processor, opsys, version)
            installer = self.toolchain_list.get(key, None)
            if installer:
                result = installer.build_and_flash(base, dirpath, command)
                return result
            else:
                print(f"ERROR: unable to find toolchain for specified device")
                exit(1)
        except Exception as e:
            print(f"ERROR running tool: {str(e)}")
            print(traceback.format_exc())
            exit(1)

    #
    # This lists the available toolchains this user can install.
    # This returns all the concrete ToolChain classes registered on this system.
    #
    def list_available(self):
        return self.toolchain_list
